@@ -5,31 +5,30 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2023 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from gi.repository import Gtk, GObject, Pango
+from gi.repository import GObject, Gtk, Pango
 
 
 class CellRendererButton(Gtk.CellRenderer):
     __gproperties__ = {
         "text": (GObject.TYPE_STRING, None, "Text",
-            "Displayed text", GObject.ParamFlags.READWRITE),
+                 "Displayed text", GObject.ParamFlags.READWRITE),
         'visible': (GObject.TYPE_INT, 'Visible',
-            'Visible', 0, 10, 0, GObject.ParamFlags.READWRITE),
+                    'Visible', 0, 10, 0, GObject.ParamFlags.READWRITE),
         'sensitive': (GObject.TYPE_INT, 'Sensitive',
-            'Sensitive', 0, 10, 0, GObject.ParamFlags.READWRITE),
-        }
+                      'Sensitive', 0, 10, 0, GObject.ParamFlags.READWRITE),
+    }
 
     __gsignals__ = {
         'clicked': (GObject.SignalFlags.RUN_LAST, GObject.TYPE_NONE,
-            (GObject.TYPE_STRING,)),
-        }
+                    (GObject.TYPE_STRING,)),
+    }
 
     def __init__(self, text=""):
         Gtk.CellRenderer.__init__(self)
@@ -91,7 +90,7 @@ class CellRendererButton(Gtk.CellRenderer):
             return (0, 0, 30, 18)
         else:
             return (cell_area.x, cell_area.y,
-                cell_area.width, cell_area.height)
+                    cell_area.width, cell_area.height)
 
     def do_start_editing(
             self, event, widget, path, background_area, cell_area, flags):
