@@ -5,25 +5,25 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2023 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gettext
 import datetime
+import gettext
 
-from gi.repository import Gdk, GObject, Gtk
-
-from dateutil.relativedelta import relativedelta
 from dateutil.parser import parse
+from dateutil.relativedelta import relativedelta
+from gi.repository import Gdk, GObject, Gtk
 
 from .common import IconFactory
 
-__all__ = ['Date', 'CellRendererDate', 'Time', 'CellRendererTime', 'DateTime']
+__all__ = [
+    'Date', 'CellRendererDate', 'Time', 'CellRendererTime', 'DateTime',
+    'date_parse']
 
 _ = gettext.gettext
 
@@ -58,20 +58,20 @@ class Date(Gtk.Entry):
     __gtype_name__ = 'Date'
     __gproperties__ = {
         'value': (GObject.TYPE_PYOBJECT,
-            _('Value'),
-            _('Displayed value'),
-            GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+                  _('Value'),
+                  _('Displayed value'),
+                  GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
         'format': (GObject.TYPE_STRING,
-            '%x',
-            _('Format'),
-            _('Display format'),
-            GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
-        }
+                   '%x',
+                   _('Format'),
+                   _('Display format'),
+                   GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+    }
     __gsignals__ = {
         'date-changed': (
             GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION,
             GObject.TYPE_NONE, ()),
-        }
+    }
 
     def __init__(self):
         self.__date = None
@@ -100,7 +100,7 @@ class Date(Gtk.Entry):
         self.__cal_popup.connect('delete-event', self.cal_popup_closed)
         self.__cal_popup.connect('key-press-event', self.cal_popup_key_pressed)
         self.__cal_popup.connect('button-press-event',
-            self.cal_popup_button_pressed)
+                                 self.cal_popup_button_pressed)
 
         self.__calendar = Gtk.Calendar()
         cal_options = (
@@ -111,7 +111,7 @@ class Date(Gtk.Entry):
         self.__cal_popup.add(self.__calendar)
         self.__calendar.connect('day-selected', self.cal_popup_changed)
         self.__calendar.connect('day-selected-double-click',
-            self.cal_popup_double_click)
+                                self.cal_popup_double_click)
         self.__calendar.show()
 
     def parse(self):
@@ -143,8 +143,9 @@ class Date(Gtk.Entry):
                 self.__date.month - 1, self.__date.year)
             self.__calendar.select_day(self.__date.day)
         self.__cal_popup.set_transient_for(self.get_toplevel())
-        popup_position(self, self.__cal_popup)
+        # Show popup before because position needs the popup allocation
         popup_show(self.__cal_popup)
+        popup_position(self, self.__cal_popup)
 
     def cal_popup_changed(self, calendar):
         year, month, day = self.__calendar.get_date()
@@ -231,11 +232,11 @@ GObject.type_register(Date)
 class CellRendererDate(Gtk.CellRendererText):
     __gproperties__ = {
         'format': (GObject.TYPE_STRING,
-            _('Format'),
-            _('Display format'),
-            '%x',
-            GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
-        }
+                   _('Format'),
+                   _('Display format'),
+                   '%x',
+                   GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+    }
 
     def __init__(self):
         self.__format = '%x'
@@ -296,20 +297,20 @@ class Time(Gtk.ComboBox):
     __gtype_name__ = 'Time'
     __gproperties__ = {
         'value': (GObject.TYPE_PYOBJECT,
-            _('Value'),
-            _('Displayed value'),
-            GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+                  _('Value'),
+                  _('Displayed value'),
+                  GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
         'format': (GObject.TYPE_STRING,
-            _('Format'),
-            _('Display format'),
-            '%X',
-            GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
-        }
+                   _('Format'),
+                   _('Display format'),
+                   '%X',
+                   GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+    }
     __gsignals__ = {
         'time-changed': (
             GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION,
             GObject.TYPE_NONE, ()),
-        }
+    }
 
     def __init__(self):
         self.__time = None
@@ -404,11 +405,11 @@ GObject.type_register(Time)
 class CellRendererTime(Gtk.CellRendererCombo):
     __gproperties__ = {
         'format': (GObject.TYPE_STRING,
-            '%X',
-            _('Format'),
-            _('Display format'),
-            GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
-        }
+                   '%X',
+                   _('Format'),
+                   _('Display format'),
+                   GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+    }
 
     def __init__(self):
         self.__format = '%X'
@@ -463,25 +464,27 @@ class DateTime(Gtk.HBox):
     __gtype_name__ = 'DateTime'
     __gproperties__ = {
         'value': (GObject.TYPE_PYOBJECT,
-            _('Value'),
-            _('Displayed value'),
-            GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+                  _('Value'),
+                  _('Displayed value'),
+                  GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
         'date-format': (GObject.TYPE_STRING,
-            '%x',
-            _('Date Format'),
-            _('Displayed date format'),
-            GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
+                        '%x',
+                        _('Date Format'),
+                        _('Displayed date format'),
+                        GObject.ParamFlags.READABLE |
+                        GObject.ParamFlags.WRITABLE),
         'time-format': (GObject.TYPE_STRING,
-            '%X',
-            _('Date Format'),
-            _('Displayed date format'),
-            GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
-        }
+                        '%X',
+                        _('Date Format'),
+                        _('Displayed date format'),
+                        GObject.ParamFlags.READABLE |
+                        GObject.ParamFlags.WRITABLE),
+    }
     __gsignals__ = {
         'datetime-changed': (
             GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION,
             GObject.TYPE_NONE, ()),
-        }
+    }
 
     def __init__(self):
         Gtk.HBox.__init__(self, spacing=4)
@@ -490,13 +493,13 @@ class DateTime(Gtk.HBox):
         self.pack_start(self.__date, expand=True, fill=True, padding=0)
         self.__date.show()
         self.__date.connect('date-changed',
-            lambda e: self.emit('datetime-changed'))
+                            lambda e: self.emit('datetime-changed'))
 
         self.__time = Time()
         self.pack_start(self.__time, expand=True, fill=True, padding=0)
         self.__time.show()
         self.__time.connect('time-changed',
-            lambda e: self.emit('datetime-changed'))
+                            lambda e: self.emit('datetime-changed'))
 
     def parse(self):
         self.__date.parse()
@@ -546,8 +549,19 @@ GObject.type_register(DateTime)
 
 def popup_position(widget, popup):
     allocation = widget.get_allocation()
+    popup_allocation = popup.get_allocation()
     x, y = widget.get_window().get_root_coords(allocation.x, allocation.y)
-    popup.move(x, y + allocation.height)
+    display = widget.get_display()
+    monitor = display.get_monitor_at_window(widget.get_window())
+    monitor_geometry = monitor.get_geometry()
+    if (monitor_geometry.height
+            < y + allocation.height + popup_allocation.height):
+        y -= popup_allocation.height
+    else:
+        y += allocation.height
+    if monitor_geometry.width < x + popup_allocation.width:
+        x -= popup_allocation.width
+    popup.move(x, y)
 
 
 def popup_show(popup):
@@ -631,7 +645,7 @@ OPERATORS = {
     Gdk.KEY_m: relativedelta(months=1),
     Gdk.KEY_Y: relativedelta(years=-1),
     Gdk.KEY_y: relativedelta(years=1),
-    }
+}
 
 if __name__ == '__main__':
     win = Gtk.Window()
