@@ -5,12 +5,11 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2023 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
@@ -22,7 +21,8 @@ import sys
 import threading
 import traceback
 import webbrowser
-from urllib.parse import urlparse, parse_qsl, unquote
+from functools import partial
+from urllib.parse import parse_qsl, unquote, urlparse
 
 from gi.repository import Gdk, GdkPixbuf, Gio, GLib, Gtk
 
@@ -31,9 +31,9 @@ import gnuhealth.plugins
 import gnuhealth.rpc as rpc
 import gnuhealth.translate as translate
 from gnuhealth.action import Action
-from gnuhealth.common import RPCExecute, RPCException, RPCContextReload
-from gnuhealth.common.cellrendererclickablepixbuf import \
-        CellRendererClickablePixbuf
+from gnuhealth.common import RPCContextReload, RPCException, RPCExecute
+from gnuhealth.common.cellrendererclickablepixbuf import (
+    CellRendererClickablePixbuf)
 from gnuhealth.config import CONFIG, GNUHEALTH_ICON, get_config_dir
 from gnuhealth.exceptions import GNUHealthError, GNUHealthServerUnavailable
 from gnuhealth.gui.window import Window
@@ -47,7 +47,6 @@ from gnuhealth.gui.window.activity import Activity
 from gnuhealth import __version__
 import ast
 
-
 _ = gettext.gettext
 logger = logging.getLogger(__name__)
 _PRIORITIES = [getattr(Gio.NotificationPriority, p)
@@ -94,9 +93,11 @@ class Main(Gtk.Application):
             CONFIG[key] = value.get_boolean()
 
         for name, key in [
+                ('save-tree-width', 'client.save_tree_width'),
                 ('save-tree-state', 'client.save_tree_state'),
+                ('code-scanner-sound', 'client.code_scanner_sound'),
                 ('spell-checking', 'client.spellcheck'),
-                ]:
+        ]:
             variant = GLib.Variant.new_boolean(CONFIG[key])
             action = Gio.SimpleAction.new_stateful(name, None, variant)
             action.connect('change-state', on_change_action_boolean, key)
@@ -116,6 +117,14 @@ class Main(Gtk.Application):
         action.connect('activate', lambda *a: self.edit_limit())
         self.add_action(action)
 
+        action = Gio.SimpleAction.new('documentation', None)
+        action.connect('activate', lambda *a: common.open_documentation())
+        self.add_action(action)
+
+        action = Gio.SimpleAction.new('donation', None)
+        action.connect('activate', lambda *a: common.open_donation())
+        self.add_action(action)
+
         self._shortcuts = None
         action = Gio.SimpleAction.new('shortcuts', None)
         action.connect('activate', lambda *a: self.shortcuts())
@@ -144,7 +153,7 @@ class Main(Gtk.Application):
         self.add_action(action)
         self.set_accels_for_action('app.gh_cli', ['<Primary><Shift>Z'])
 
-        # Main Menu
+        # GNU Health Block: Set Main Menu menubar
         main_menu = Gio.Menu.new()
         file_menu = Gio.Menu.new()
         settings_menu = Gio.Menu.new()
@@ -167,12 +176,17 @@ class Main(Gtk.Application):
         form.append(_("Save Width/Height"), 'app.save-width-height')
         form.append(_("Save Tree State"), 'app.save-tree-state')
         form.append(_("Spell Checking"), 'app.spell-checking')
+        form.append(_("Play Sound for Code Scanner"), 'app.code-scanner-sound')
 
         settings_menu.append(_("Search Limit..."), 'app.search-limit')
         settings_menu.append(_("Email..."), 'app.email')
 
+        help_menu.append(_("Documentation..."), 'app.documentation')
         help_menu.append(_("Keyboard Shortcuts..."), 'app.shortcuts')
         help_menu.append(_("About..."), 'app.about')
+        help_menu.append(
+            _("Donate to GNU Health") + " " + "\N{heavy black heart}",
+            'app.donation')
 
         file_menu.append(_("Quit"), 'app.quit')
 
@@ -183,11 +197,10 @@ class Main(Gtk.Application):
             self.window.present()
             return
 
-        self.window = Gtk.ApplicationWindow(
-            application=self)
+        self.window = Gtk.ApplicationWindow(application=self)
         title = self.get_main_title()
         self.window.set_title(title)
-        self.window.set_default_size(960, 720)
+        self.window.set_default_size(1280, 960)
         self.window.maximize()
         self.window.set_position(Gtk.WindowPosition.CENTER)
         self.window.set_resizable(True)
@@ -196,16 +209,13 @@ class Main(Gtk.Application):
         self.window.connect("delete_event", self.on_quit)
         common.setup_window(self.window)
 
-        self.header = Gtk.HeaderBar.new()
-        self.header.set_show_close_button(True)
+        # GNU Health
         # Disable title bar
         # self.window.set_titlebar(self.header)
+        self.header = Gtk.HeaderBar.new()
+        self.header.set_show_close_button(True)
         self.set_title()
 
-        self.primary_menu = Gtk.MenuButton.new()
-        self.primary_menu.set_menu_model(self._get_primary_menu())
-        self.header.pack_end(self.primary_menu)
-
         menu = Gtk.Button.new()
         menu.set_relief(Gtk.ReliefStyle.NONE)
         menu.set_image(
@@ -217,14 +227,14 @@ class Main(Gtk.Application):
         favorite = Gtk.MenuButton.new()
         favorite.set_relief(Gtk.ReliefStyle.NONE)
         favorite.set_image(common.IconFactory.get_image(
-                'gnuhealth-bookmarks', Gtk.IconSize.BUTTON))
+            'gnuhealth-bookmarks', Gtk.IconSize.BUTTON))
         self.menu_favorite = Gtk.Menu.new()
         favorite.set_popup(self.menu_favorite)
         favorite.connect('button-press-event', self.favorite_set)
-        self.header.pack_start(favorite)
+        # self.header.pack_start(favorite)
 
-        self.set_global_search()
-        self.header.pack_start(self.global_search_entry)
+        # self.set_global_search()
+        # self.header.pack_start(self.global_search_entry)
 
         self.accel_group = Gtk.AccelGroup()
         self.window.add_accel_group(self.accel_group)
@@ -240,9 +250,11 @@ class Main(Gtk.Application):
             '<gnuhealth>/Form/Delete',
             Gdk.KEY_D, Gdk.ModifierType.CONTROL_MASK)
         Gtk.AccelMap.add_entry(
-            '<gnuhealth>/Form/Next', Gdk.KEY_Page_Down, 0)
+            '<gnuhealth>/Form/Next',
+            Gdk.KEY_Down, Gdk.ModifierType.CONTROL_MASK)
         Gtk.AccelMap.add_entry(
-            '<gnuhealth>/Form/Previous', Gdk.KEY_Page_Up, 0)
+            '<gnuhealth>/Form/Previous',
+            Gdk.KEY_Up, Gdk.ModifierType.CONTROL_MASK)
         Gtk.AccelMap.add_entry(
             '<gnuhealth>/Form/Switch View', Gdk.KEY_L,
             Gdk.ModifierType.CONTROL_MASK)
@@ -284,12 +296,6 @@ class Main(Gtk.Application):
 
         self.info = Gtk.VBox()
         self.vbox.pack_start(self.info, expand=False, fill=True, padding=0)
-        if CONFIG['client.check_version']:
-            common.check_version(self.info)
-            GLib.timeout_add_seconds(
-                int(CONFIG['download.frequency']), common.check_version,
-                self.info)
-
         self.pane = Gtk.HPaned()
         self.vbox.pack_start(self.pane, expand=True, fill=True, padding=0)
         self.pane.set_position(int(CONFIG['menu.pane']))
@@ -325,11 +331,14 @@ class Main(Gtk.Application):
 
         self.set_title()  # Adds username/profile while password is asked
         try:
-            common.Login()
+            common.get_credentials()
         except Exception as exception:
-            if (not isinstance(exception, GNUHealthError)
-                    or exception.faultCode != 'QueryCanceled'):
+            if not isinstance(exception, GNUHealthError):
                 common.error(exception, traceback.format_exc())
+            elif exception.faultCode == 'SessionFailed':
+                common.message(
+                    _("Could not get a session."),
+                    msg_type=Gtk.MessageType.ERROR)
             return self.quit()
         self.get_preferences()
 
@@ -338,17 +347,6 @@ class Main(Gtk.Application):
         msg = "Connected to GNU Health Server"
         self.activity_log_entry(msg, 'info')
 
-    def get_main_title(self):
-        user = CONFIG['login.login']
-        host = CONFIG['login.host']
-        db = CONFIG['login.db']
-        conn_info = f'{user}@{host}/{db}'
-        if CONFIG['client.title']:
-            res = f"{CONFIG['client.title']} - {conn_info}"
-        else:
-            res = conn_info
-        return res
-        
     def gnuhealth_cli(self):
         self.cli.grab_focus()
 
@@ -379,35 +377,6 @@ class Main(Gtk.Application):
         super().add_window(window)
         common.setup_window(window)
 
-    def _get_primary_menu(self):
-        menu = Gio.Menu.new()
-        menu.append(_("Preferences..."), 'app.preferences')
-
-        section = Gio.Menu.new()
-        toolbar = Gio.Menu.new()
-        section.append_submenu(_("Toolbar"), toolbar)
-        toolbar.append(_("Default"), 'app.toolbar::default')
-        toolbar.append(_("Text and Icons"), 'app.toolbar::both')
-        toolbar.append(_("Text"), 'app.toolbar::text')
-        toolbar.append(_("Icons"), 'app.toolbar::icons')
-
-        form = Gio.Menu.new()
-        section.append_submenu(_("Form"), form)
-        form.append(_("Save Column Width"), 'app.save-tree-width')
-        form.append(_("Save Tree State"), 'app.save-tree-state')
-        form.append(_("Spell Checking"), 'app.spell-checking')
-
-        section.append(_("Search Limit..."), 'app.search-limit')
-        section.append(_("Check Version"), 'app.check-version')
-
-        menu.append_section(_("Options"), section)
-
-        section = Gio.Menu.new()
-        section.append(_("Keyboard Shortcuts..."), 'app.shortcuts')
-        section.append(_("About..."), 'app.about')
-        menu.append_section(_("Help"), section)
-        return menu
-
     def set_global_search(self):
         self.global_search_entry = Gtk.Entry.new()
         self.global_search_entry.set_width_chars(20)
@@ -422,6 +391,7 @@ class Main(Gtk.Application):
         global_search_completion.pack_start(text_cell, expand=True)
         global_search_completion.add_attribute(text_cell, "markup", 1)
         global_search_completion.props.popup_set_width = True
+
         self.global_search_entry.set_completion(global_search_completion)
 
         def match_selected(completion, model, iter_):
@@ -429,14 +399,15 @@ class Main(Gtk.Application):
             if model == self.menu_screen.model_name:
                 # ids is not defined to prevent to add suffix
                 Action.exec_keyword('tree_open', {
-                        'model': model,
-                        'id': record_id,
-                        })
+                    'model': model,
+                    'id': record_id,
+                })
             else:
-                Window.create(model,
-                              res_id=record_id,
-                              mode=['form', 'tree'],
-                              name=model_name)
+                Window.create(
+                    model,
+                    res_id=record_id,
+                    mode=['form', 'tree'],
+                    name=model_name)
             self.global_search_entry.set_text('')
             return True
 
@@ -484,9 +455,10 @@ class Main(Gtk.Application):
                 widget.emit('changed')
                 end()
 
-            RPCExecute('model', 'ir.model', 'global_search', search_text,
-                       CONFIG['client.limit'], self.menu_screen.model_name,
-                       context=self.menu_screen.context, callback=set_result)
+            RPCExecute(
+                'model', 'ir.model', 'global_search', search_text,
+                CONFIG['client.limit'], self.menu_screen.model_name,
+                context=self.menu_screen.context, callback=set_result)
             return False
 
         def changed(widget):
@@ -538,24 +510,26 @@ class Main(Gtk.Application):
 
         def _action_favorite(widget, id_):
             event = Gtk.get_current_event()
-            allow_similar = (event.state & Gdk.ModifierType.MOD1_MASK
-                             or event.state & Gdk.ModifierType.SHIFT_MASK)
+            allow_similar = (
+                event.state & Gdk.ModifierType.MOD1_MASK
+                or event.state & Gdk.ModifierType.SHIFT_MASK)
             with Window(allow_similar=allow_similar):
                 # ids is not defined to prevent to add suffix
                 Action.exec_keyword('tree_open', {
-                        'model': self.menu_screen.model_name,
-                        'id': id_,
-                        })
+                    'model': self.menu_screen.model_name,
+                    'id': id_,
+                })
 
         def _manage_favorites(widget):
-            Window.create(self.menu_screen.model_name + '.favorite',
-                          mode=['tree', 'form'],
-                          name=_("Favorites"))
+            Window.create(
+                self.menu_screen.model_name + '.favorite',
+                mode=['tree', 'form'],
+                name=_("Favorites"))
         try:
-            favorites = RPCExecute('model',
-                                   self.menu_screen.model_name +
-                                   '.favorite', 'get',
-                                   process_exception=False)
+            favorites = RPCExecute(
+                'model',
+                self.menu_screen.model_name + '.favorite', 'get',
+                process_exception=False)
         except Exception:
             return False
         for id_, name, icon in favorites:
@@ -563,8 +537,9 @@ class Main(Gtk.Application):
             menuitem.connect('activate', _action_favorite, id_)
             self.menu_favorite.add(menuitem)
         self.menu_favorite.add(Gtk.SeparatorMenuItem())
-        manage_favorites = Gtk.MenuItem(label=_("Manage..."),
-                                        use_underline=True)
+        manage_favorites = Gtk.MenuItem(
+            label=_("Manage..."),
+            use_underline=True)
         manage_favorites.connect('activate', _manage_favorites)
         self.menu_favorite.add(manage_favorites)
         self.menu_favorite.show_all()
@@ -587,7 +562,8 @@ class Main(Gtk.Application):
             barstyle = Gtk.ToolbarStyle.ICONS
         for page_idx in range(self.notebook.get_n_pages()):
             page = self.get_page(page_idx)
-            page.toolbar.set_style(barstyle)
+            if page.toolbar:
+                page.toolbar.set_style(barstyle)
 
     def edit_limit(self):
         from gnuhealth.gui.window.limit import Limit
@@ -610,18 +586,25 @@ class Main(Gtk.Application):
         except RPCException:
             prefs = {}
 
-        threads = []
-        for target in (
-                common.IconFactory.load_icons,
-                common.MODELACCESS.load_models,
-                common.MODELHISTORY.load_history,
-                common.VIEW_SEARCH.load_searches,
-                ):
-            t = threading.Thread(target=target)
-            threads.append(t)
-            t.start()
-        for t in threads:
-            t.join()
+        targets = [
+            common.IconFactory.load_icons,
+            common.MODELACCESS.load_models,
+            common.MODELHISTORY.load_history,
+            common.MODELNOTIFICATION.load_names,
+            common.VIEW_SEARCH.load_searches,
+        ]
+        if CONFIG['thread']:
+            threads = []
+            for target in targets:
+                t = threading.Thread(target=target)
+                threads.append(t)
+                t.start()
+            for t in threads:
+                t.join()
+        else:
+            for target in targets:
+                target()
+
         if prefs and 'language_direction' in prefs:
             translate.set_language_direction(prefs['language_direction'])
             CONFIG['client.language_direction'] = \
@@ -632,13 +615,10 @@ class Main(Gtk.Application):
         self.set_title(prefs.get('status_bar', ''))
         if prefs and 'language' in prefs:
             translate.setlang(prefs['language'], prefs.get('locale'))
-            if CONFIG['client.lang'] != prefs['language']:
-                self.favorite_unset()
-                self.primary_menu.set_menu_model(self._get_primary_menu())
             CONFIG['client.lang'] = prefs['language']
         common.MODELNAME.clear()
         # Set placeholder after language is set to get correct translation
-        self.global_search_entry.set_placeholder_text(_("Action"))
+        self.global_search_entry.set_placeholder_text(_("Global Search"))
         CONFIG.save()
 
     def preferences(self):
@@ -648,9 +628,9 @@ class Main(Gtk.Application):
         Preference(rpc._USER, self.get_preferences)
 
     def sig_win_close(self, widget=None):
-        self._sig_remove_book(widget,
-                              self.notebook.get_nth_page(
-                                  self.notebook.get_current_page()))
+        self._sig_remove_book(
+            widget,
+            self.notebook.get_nth_page(self.notebook.get_current_page()))
 
     def close_pages(self):
         if self.notebook.get_n_pages():
@@ -703,7 +683,7 @@ class Main(Gtk.Application):
                 ('app.tab-next', _("Next tab")),
                 ('app.shortcuts', _("Shortcuts")),
                 ('app.quit', _("Quit")),
-                ]:
+        ]:
             shortcut = Gtk.ShortcutsShortcut()
             shortcut.props.title = title
             accels = self.get_accels_for_action(action)
@@ -715,58 +695,62 @@ class Main(Gtk.Application):
 
         for name, title, groups in [
                 ('entry', _("Edition Shortcuts"), [
-                        (_("Text Entries"), [
-                                ('<Primary>x', _("Cut selected text")),
-                                ('<Primary>c', _("Copy selected text")),
-                                ('<Primary>v', _("Paste copied text")),
-                                ('Tab', _("Next entry")),
-                                ('<Shift>Tab', _("Previous entry")),
-                                ]),
-                        (_("Relation Entries"), [
-                                ('F3', _("Create new relation")),
-                                ('F2', _("Open/Search relation")),
-                                ]),
-                        (_("List Entries"), [
-                                ('F4', _("Switch view")),
-                                ('F3', _("Create/Select new line")),
-                                ('F2', _("Open relation")),
-                                ('Delete',
-                                    _("Mark line for deletion/removal")),
-                                ('<Ctrl>Delete', _("Mark line for removal")),
-                                ('Insert', _("Unmark line for deletion")),
-                                ]),
-                        ]),
+                    (_("Text Entries"), [
+                        ('<Primary>x', _("Cut selected text")),
+                        ('<Primary>c', _("Copy selected text")),
+                        ('<Primary>v', _("Paste copied text")),
+                        ('Tab', _("Next entry")),
+                        ('<Shift>Tab', _("Previous entry")),
+                    ]),
+                    (_("Relation Entries"), [
+                        ('F3', _("Create new relation")),
+                        ('F2', _("Open/Search relation")),
+                    ]),
+                    (_("List Entries"), [
+                        ('F4', _("Switch view")),
+                        ('F3', _("Create/Select new line")),
+                        ('F2', _("Open relation")),
+                        ('Delete',
+                         _("Mark line for deletion/removal")),
+                        ('<Ctrl>Delete', _("Mark line for removal")),
+                        ('Insert', _("Unmark line for deletion")),
+                    ]),
+                ]),
                 ('tree', _("List/Tree Shortcuts"), [
-                        (_("Move Cursor"), [
-                                ('Right', _("Move right")),
-                                ('Left', _("Move left")),
-                                ('Up', _("Move up")),
-                                ('Down', _("Move down")),
-                                ('Page_Up', _("Move up of one page")),
-                                ('Page_Down', _("Move down of one page")),
-                                ('Home', _("Move to top")),
-                                ('End', _("Move to bottom")),
-                                ('BackSpace', _("Move to parent")),
-                                ]),
-                        (_("Selection"), [
-                                ('<Ctrl>a <Ctrl>slash', _("Select all")),
-                                ('<Shift><Ctrl>a <Shift><Ctrl>slash',
-                                    _("Unselect all")),
-                                ('BackSpace', _("Select parent")),
-                                ('space', _("Select/Activate current row")),
-                                ('<Shift>space Return',
-                                    _("Select/Activate current row")),
-                                ('<Ctrl>space', _("Toggle selection")),
-                                ]),
-                        (_("Expand/Collapse"), [
-                                ('plus', _("Expand row")),
-                                ('minus', _("Collapse row")),
-                                ('space', _("Toggle row")),
-                                ('<Shift>Left', _("Collapse all rows")),
-                                ('<Shift>Right', _("Expand all rows")),
-                                ]),
-                        ]),
-                ]:
+                    (_("Move Cursor"), [
+                        ('Right', _("Move right")),
+                        ('Left', _("Move left")),
+                        ('Up', _("Move up")),
+                        ('Down', _("Move down")),
+                        ('Page_Up', _("Move up of one page")),
+                        ('Page_Down', _("Move down of one page")),
+                        ('Home', _("Move to top")),
+                        ('End', _("Move to bottom")),
+                        ('BackSpace', _("Move to parent")),
+                    ]),
+                    (_("Edition"), [
+                        ('<Primary>c', _("Copy selected rows")),
+                        ('<Primary>v', _("Insert copied rows")),
+                    ]),
+                    (_("Selection"), [
+                        ('<Ctrl>a <Ctrl>slash', _("Select all")),
+                        ('<Shift><Ctrl>a <Shift><Ctrl>slash',
+                         _("Unselect all")),
+                        ('BackSpace', _("Select parent")),
+                        ('space', _("Select/Activate current row")),
+                        ('<Shift>space Return',
+                         _("Select/Activate current row")),
+                        ('<Ctrl>space', _("Toggle selection")),
+                    ]),
+                    (_("Expand/Collapse"), [
+                        ('plus', _("Expand row")),
+                        ('minus', _("Collapse row")),
+                        ('space', _("Toggle row")),
+                        ('<Shift>Left', _("Collapse all rows")),
+                        ('<Shift>Right', _("Expand all rows")),
+                    ]),
+                ]),
+        ]:
             section = Gtk.ShortcutsSection()
             section.props.section_name = name
             section.props.title = title
@@ -801,21 +785,22 @@ class Main(Gtk.Application):
 
     def menu_row_activate(self):
         screen = self.menu_screen
-        record_id = (screen.current_record.id
-                     if screen.current_record else None)
+        record_id = (
+            screen.current_record.id
+            if screen.current_record else None)
         # ids is not defined to prevent to add suffix
         return Action.exec_keyword('tree_open', {
-                'model': screen.model_name,
-                'id': record_id,
-                }, warning=False)
+            'model': screen.model_name,
+            'id': record_id,
+        }, warning=False)
 
     def sig_win_menu(self, prefs=None):
         from gnuhealth.gui.window.view_form.screen import Screen
 
         if not prefs:
             try:
-                prefs = RPCExecute(
-                    'model', 'res.user', 'get_preferences', False)
+                prefs = RPCExecute('model', 'res.user', 'get_preferences',
+                                   False)
             except RPCException:
                 return False
 
@@ -834,9 +819,10 @@ class Main(Gtk.Application):
         decoder = PYSONDecoder(ctx)
         action_ctx = decoder.decode(action.get('pyson_context') or '{}')
         domain = decoder.decode(action['pyson_domain'])
-        screen = Screen(action['res_model'], mode=['tree'], view_ids=view_ids,
-                        domain=domain, context=action_ctx, readonly=True,
-                        limit=None, row_activate=self.menu_row_activate)
+        screen = Screen(
+            action['res_model'], mode=['tree'], view_ids=view_ids,
+            domain=domain, context=action_ctx, readonly=True, limit=None,
+            row_activate=self.menu_row_activate)
         # Use alternate view to not show search box
         screen.screen_container.alternate_view = True
         screen.switch_view(view_type=screen.current_view.view_type)
@@ -875,7 +861,8 @@ class Main(Gtk.Application):
             if treeview.props.window:
                 self.toggle_favorite(renderer, path, treeview)
         favorite_renderer.connect(
-            'clicked', lambda *a: GLib.idle_add(toggle_favorite, *a), treeview)
+            'clicked',
+            lambda *a: GLib.idle_add(toggle_favorite, *a), treeview)
         # Unset fixed height mode to add column
         treeview.set_fixed_height_mode(False)
         treeview.set_property(
@@ -934,7 +921,7 @@ class Main(Gtk.Application):
                 expand=False, fill=False, padding=0)
         name = page.name
         label = Gtk.Label(
-            label=common.ellipsize(name, 20),
+            label=common.ellipsize(name.split(' / ')[-1], 20),
             halign=Gtk.Align.START)
         self.tooltips.set_tip(label, page.name)
         self.tooltips.enable()
@@ -944,7 +931,7 @@ class Main(Gtk.Application):
         button.set_relief(Gtk.ReliefStyle.NONE)
         button.set_can_focus(False)
         button.add(common.IconFactory.get_image(
-                'gnuhealth-close', Gtk.IconSize.MENU))
+            'gnuhealth-close', Gtk.IconSize.MENU))
         self.tooltips.set_tip(button, _('Close Tab'))
         button.connect('clicked', self._sig_remove_book, page.widget)
         hbox.pack_start(button, expand=False, fill=False, padding=0)
@@ -982,7 +969,6 @@ class Main(Gtk.Application):
             for i in range(len(self.pages)):
                 if self.pages[i].widget == page_widget:
                     page = self.pages.pop(i)
-                    page.signal_unconnect(self)
                     break
             self.notebook.remove_page(page_id)
 
@@ -994,7 +980,7 @@ class Main(Gtk.Application):
                 if i.widget == page_widget:
                     if self.previous_pages[i]:
                         next_page_id = self.notebook.page_num(
-                                self.previous_pages[i])
+                            self.previous_pages[i])
                     to_pop.append(i)
             to_pop.reverse()
             for i in to_pop:
@@ -1043,6 +1029,7 @@ class Main(Gtk.Application):
             dialog.show()
 
     def _open_url(self, url):
+        loads = partial(json.loads, object_hook=object_hook)
         urlp = urlparse(url)
         if not urlp.scheme == 'gnuhealth':
             return
@@ -1056,95 +1043,81 @@ class Main(Gtk.Application):
         if urlp.params:
             try:
                 params.update(dict(parse_qsl(urlp.params,
-                                             strict_parsing=True)))
+                              strict_parsing=True)))
             except ValueError:
                 return
 
         def open_model(path):
+            attributes = {}
             model, path = (path.split('/', 1) + [''])[:2]
             if not model:
                 return
-            res_id = None
-            mode = None
+            attributes = {}
             try:
-                view_ids = json.loads(params.get('views', '[]'))
-                limit = json.loads(params.get('limit', 'null'))
-                name = json.loads(params.get('name', '""'))
-                search_value = json.loads(params.get('search_value', '[]'),
-                                          object_hook=object_hook)
-                domain = json.loads(params.get('domain', '[]'),
-                                    object_hook=object_hook)
-                context = json.loads(params.get('context', '{}'),
-                                     object_hook=object_hook)
-                context_model = params.get('context_model')
-                tab_domain = json.loads(params.get('tab_domain', '[]'),
-                                        object_hook=object_hook)
+                attributes['view_ids'] = loads(params.get('views', '[]'))
+                if 'limit' in params:
+                    attributes['limit'] = loads(params.get('limit', 'null'))
+                attributes['name'] = loads(params.get('name', '""'))
+                attributes['search_value'] = loads(
+                    params.get('search_value', '[]'))
+                attributes['domain'] = loads(params.get('domain', '[]'))
+                attributes['context'] = loads(params.get('context', '{}'))
+                attributes['context_model'] = params.get('context_model')
+                attributes['tab_domain'] = loads(
+                    params.get('tab_domain', '[]'))
             except ValueError:
                 return
             if path:
                 try:
-                    res_id = int(path)
+                    attributes['res_id'] = int(path)
                 except ValueError:
                     return
-                mode = ['form', 'tree']
+                attributes['mode'] = ['form', 'tree']
             try:
-                Window.create(
-                    model,
-                    view_ids=view_ids,
-                    res_id=res_id,
-                    domain=domain,
-                    context=context,
-                    context_model=context_model,
-                    mode=mode,
-                    name=name,
-                    limit=limit,
-                    search_value=search_value,
-                    tab_domain=tab_domain)
+                Window.create(model, **attributes)
             except Exception:
                 # Prevent crashing the client
                 return
 
         def open_wizard(wizard):
+            attributes = {}
             if not wizard:
                 return
             try:
-                data = json.loads(params.get('data', '{}'),
-                                  object_hook=object_hook)
-                direct_print = json.loads(params.get('direct_print', 'false'))
-                name = json.loads(params.get('name', '""'))
-                window = json.loads(params.get('window', 'false'))
-                context = json.loads(params.get('context', '{}'),
-                                     object_hook=object_hook)
+                attributes['data'] = loads(params.get('data', '{}'))
+                attributes['direct_print'] = loads(
+                    params.get('direct_print', 'false'))
+                attributes['name'] = loads(params.get('name', '""'))
+                attributes['window'] = loads(params.get('window', 'false'))
+                attributes['context'] = loads(params.get('context', '{}'))
             except ValueError:
                 return
             try:
-                Window.create_wizard(
-                    wizard, data, direct_print=direct_print, name=name,
-                    context=context, window=window)
+                Window.create_wizard(wizard, **attributes)
             except Exception:
                 # Prevent crashing the client
                 return
 
         def open_report(report):
+            attributes = {}
             if not report:
                 return
             try:
-                data = json.loads(params.get('data'), object_hook=object_hook)
-                direct_print = json.loads(params.get('direct_print', 'false'))
-                context = json.loads(params.get('context', '{}'),
-                                     object_hook=object_hook)
+                attributes['data'] = loads(params.get('data', '{}'))
+                attributes['direct_print'] = loads(
+                    params.get('direct_print', 'false'))
+                attributes['context'] = loads(params.get('context', '{}'))
             except ValueError:
                 return
             try:
-                Action.exec_report(
-                    report, data, direct_print=direct_print, context=context)
+                Action.exec_report(report, **attributes)
             except Exception:
                 # Prevent crashing the client
                 return
 
         def open_url():
             try:
-                url = json.loads(params.get('url', 'false'))
+                url = loads(params.get('url', 'false'))
             except ValueError:
                 return
             if url:
@@ -1174,7 +1147,19 @@ class Main(Gtk.Application):
         if sys.platform != 'win32' or GLib.glib_version >= (2, 57, 0):
             self.send_notification(None, notification)
 
-# GNUHEALTH block
+    # GNU Health client
+    def get_main_title(self):
+        user = CONFIG['login.login']
+        host = CONFIG['login.host']
+        db = CONFIG['login.db']
+        conn_info = f'{user}@{host}/{db}'
+        if CONFIG['client.title']:
+            res = f"{CONFIG['client.title']} - {conn_info}"
+        else:
+            res = conn_info
+        return res
+
+    # GNUHEALTH block
     # Initialize GNU Health environment
     def init_gnuhealth_env(self):
         # Init GNU Health CLI
@@ -1210,8 +1195,6 @@ class Main(Gtk.Application):
         # Set GNU Health Footer
         self.set_ghfooter()
 
-        cli_position = self.cli_position
-
         # Include the side menu toggle
         side_menu = Gtk.Button.new()
         side_menu.set_image(
@@ -1224,24 +1207,20 @@ class Main(Gtk.Application):
         favorite = Gtk.MenuButton.new()
         favorite.set_relief(Gtk.ReliefStyle.NONE)
         favorite.set_image(common.IconFactory.get_image(
-                'gnuhealth-bookmarks', Gtk.IconSize.BUTTON))
+            'gnuhealth-bookmarks', Gtk.IconSize.BUTTON))
         self.menu_favorite = Gtk.Menu.new()
         favorite.set_popup(self.menu_favorite)
         favorite.connect('clicked', self.favorite_set)
 
         self.set_global_search()
 
-        # Default CLI position at top
-        if (not cli_position or cli_position == 'top'):
-            self.ghheader_contents.add(self.cli)
-            self.ghheader_contents.attach_next_to(side_menu, self.cli,
-                                                  Gtk.PositionType.RIGHT, 1, 1)
-
-        else:
-            self.ghheader_contents.add(side_menu)
-
-        self.ghheader_contents.attach_next_to(favorite, side_menu,
+        # CLI position at top
+        self.ghheader_contents.add(side_menu)
+        self.ghheader_contents.attach_next_to(self.cli, side_menu,
                                               Gtk.PositionType.RIGHT, 1, 1)
+        self.ghheader_contents.attach_next_to(favorite, self.cli,
+                                              Gtk.PositionType.RIGHT, 1, 1)
+
         self.ghheader_contents.attach_next_to(
             self.global_search_entry, favorite, Gtk.PositionType.RIGHT, 1, 1)
 
@@ -1260,13 +1239,7 @@ class Main(Gtk.Application):
         statusbar = self.statusbar
         ghfooter_contents = self.ghfooter_contents
 
-        if (self.cli_position == 'bottom'):
-            ghfooter_contents.add(cli)
-            ghfooter_contents.attach_next_to(statusbar, cli,
-                                             Gtk.PositionType.RIGHT, 1, 1)
-
-        else:
-            ghfooter_contents.add(statusbar)
+        ghfooter_contents.add(statusbar)
 
         self.ghfooter.pack_start(ghfooter_contents, expand=True, fill=True,
                                  padding=0)
@@ -1346,8 +1319,8 @@ class Main(Gtk.Application):
         # Get relevant environment variables
 
         info = f"{info} {pversion}\n {client_version}\n" \
-               f"{os_header} {os_info}\n" \
-               f"Platform / Kernel Info: {str(uname)}"
+            f"{os_header} {os_version} {os_info}\n" \
+            f"Platform / Kernel Info: {str(uname)}"
 
         return info
 
@@ -1355,7 +1328,7 @@ class Main(Gtk.Application):
 
         cmd = command.split()[0]
         args = command.split()[1:]
-        domain_name = domain = None
+        domain_name = None
         search_string = '%'
 
         res = RPCExecute('model', 'gnuhealth.command', 'search_read',
@@ -1379,7 +1352,7 @@ class Main(Gtk.Application):
                 domain=domain_name,
                 limit=CONFIG['client.limit'],
                 search_value=[('rec_name', 'ilike', search_string)]
-                )
+            )
 
         else:
             common.message(_('Command not found.'))
@@ -1390,4 +1363,3 @@ class Main(Gtk.Application):
             Activity().activity_window.hide()
         else:
             Activity().activity_window.show_all()
- 
