@@ -5,15 +5,11 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2023 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
-# This file is part of GNU Health.  The COPYRIGHT file at the top level of
-# this repository contains the full copyright notices and license terms.
-
 import gettext
 from gi.repository import Gtk
 
@@ -703,6 +699,21 @@ the library.  If this is what you want to do, use the GNU Lesser General
 Public License instead of this License.  But first, please read
 <http://www.gnu.org/philosophy/why-not-lgpl.html>.'''
 
+CREDITS = [
+    'The Tryton and GNU Health community',
+    'Bertrand Chenal <bertrand.chenal@b2ck.com>',
+    'Cédric Krier <cedric.krier@b2ck.com>',
+    'Franz Wiesinger',
+    'Hartmut Goebel',
+    'Korbinian Preisler <info@virtual-things.biz>',
+    'Luis Falcon <falcon@gnuhealth.org>',
+    'Mathias Behrle <info@m9s.biz>',
+    'Maxime Richez <maxime.richez@saluc.com>',
+    'Nicolas Évrard <nicolas.evrard@b2ck.com>',
+    'Sednacom <contact@sednacom.fr>',
+    'Udo Spallek <info@virtual-things.biz>',
+]
+
 
 class About(object):
 
@@ -710,10 +721,11 @@ class About(object):
         parent = get_toplevel_window()
         self.win = Gtk.AboutDialog()
         self.win.set_transient_for(parent)
-        self.win.set_name(CONFIG['client.title'])
+        self.win.set_program_name(CONFIG['client.title'])
         self.win.set_version(__version__)
         self.win.set_comments(POWERED_BY)
         self.win.set_license(LICENSE)
+        self.win.set_authors(CREDITS)
         self.win.set_website('http://www.gnuhealth.org/')
         self.win.set_logo(GH_ABOUT)
 
