@@ -5,32 +5,34 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2023 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
 import configparser
-import os
 import gettext
-import threading
 import logging
+import os
+import shutil
+import tempfile
+import threading
 
 from gi.repository import GLib, GObject, Gtk
 
-from gnuhealth import __version__, SERVER_VERSION
 import gnuhealth.common as common
-from gnuhealth.config import CONFIG, GNUHEALTH_ICON, PIXMAPS_DIR, BANNER, \
-     get_config_dir
 import gnuhealth.rpc as rpc
+from gnuhealth import __version__, SERVER_VERSION
 from gnuhealth.common.underline import set_underline
+from gnuhealth.config import CONFIG, PIXMAPS_DIR, GNUHEALTH_ICON, \
+    BANNER, get_config_dir
 
 _ = gettext.gettext
 logger = logging.getLogger(__name__)
+DEMO_HOSTNAME = 'federation.gnuhealth.org'
 
 
 class DBListEditor(object):
@@ -62,8 +64,8 @@ class DBListEditor(object):
         self.cell.connect('editing-started', self.edit_started)
         self.profile_tree = Gtk.TreeView()
         self.profile_tree.set_model(profile_store)
-        self.profile_tree.insert_column_with_attributes(-1, _('Profile'),
-            self.cell, text=0)
+        self.profile_tree.insert_column_with_attributes(
+            -1, _('Profile'), self.cell, text=0)
         self.profile_tree.connect('cursor-changed', self.profile_selected)
         scroll = Gtk.ScrolledWindow()
         scroll.set_policy(
@@ -71,12 +73,12 @@ class DBListEditor(object):
         scroll.add(self.profile_tree)
         self.add_button = Gtk.Button()
         self.add_button.set_image(common.IconFactory.get_image(
-                'gnuhealth-add', Gtk.IconSize.BUTTON))
+            'gnuhealth-add', Gtk.IconSize.BUTTON))
         tooltips.set_tip(self.add_button, _("Add new profile"))
         self.add_button.connect('clicked', self.profile_create)
         self.remove_button = Gtk.Button()
         self.remove_button.set_image(common.IconFactory.get_image(
-                'gnuhealth-remove', Gtk.IconSize.BUTTON))
+            'gnuhealth-remove', Gtk.IconSize.BUTTON))
         tooltips.set_tip(self.remove_button, _("Remove selected profile"))
         self.remove_button.get_style_context().add_class(
             Gtk.STYLE_CLASS_DESTRUCTIVE_ACTION)
@@ -104,8 +106,8 @@ class DBListEditor(object):
             use_underline=True, halign=Gtk.Align.END)
         self.database_entry = Gtk.Entry()
         self.database_entry.connect('changed', self.dbentry_changed)
-        self.database_entry.connect('changed', self.update_profiles,
-            'database')
+        self.database_entry.connect(
+            'changed', self.update_profiles, 'database')
         self.database_entry.set_activates_default(True)
         self.database_label = Gtk.Label(valign=Gtk.Align.START)
         self.database_label.set_use_markup(True)
@@ -135,8 +137,8 @@ class DBListEditor(object):
             label=set_underline(_('Username:')),
             use_underline=True, halign=Gtk.Align.END)
         self.username_entry = Gtk.Entry(hexpand=True)
-        self.username_entry.connect('changed', self.update_profiles,
-            'username')
+        self.username_entry.connect(
+            'changed', self.update_profiles, 'username')
         username.set_mnemonic_widget(self.username_entry)
         self.username_entry.set_activates_default(True)
         grid.attach(username, 0, 3, 1, 1)
@@ -199,8 +201,8 @@ class DBListEditor(object):
         model = self.profile_tree.get_model()
         model.append(['', False])
         column = self.profile_tree.get_column(0)
-        self.profile_tree.set_cursor(len(model) - 1, column,
-            start_editing=True)
+        self.profile_tree.set_cursor(
+            len(model) - 1, column, start_editing=True)
         self.db_cache = None
 
     def profile_delete(self, button):
@@ -226,8 +228,8 @@ class DBListEditor(object):
         for field in fields:
             entry = getattr(self, '%s_entry' % field)
             try:
-                entry_value = self.profiles.get(self.current_profile['name'],
-                    field)
+                entry_value = self.profiles.get(
+                    self.current_profile['name'], field)
             except configparser.NoOptionError:
                 entry_value = ''
             entry.set_text(entry_value)
@@ -238,8 +240,8 @@ class DBListEditor(object):
 
     def edit_started(self, renderer, editable, path):
         if isinstance(editable, Gtk.Entry):
-            editable.connect('focus-out-event', self.edit_profilename,
-                renderer, path)
+            editable.connect(
+                'focus-out-event', self.edit_profilename, renderer, path)
 
     def edit_profilename(self, editable, event, renderer, path):
         newtext = editable.get_text()
@@ -281,7 +283,8 @@ class DBListEditor(object):
         model, selection = self.profile_tree.get_selection().get_selected()
         if not selection:
             return
-        active = all(self.profiles.has_option(profile_name, option)
+        active = all(self.profiles.has_option(
+            profile_name, option)
             for option in ('host', 'database'))
         model[selection][1] = active
 
@@ -298,9 +301,13 @@ class DBListEditor(object):
 
     def refresh_databases(self, host, port):
         self.dbs_updated = threading.Event()
-        threading.Thread(target=self.refresh_databases_start,
-            args=(host, port)).start()
-        GLib.timeout_add(100, self.refresh_databases_end, host, port)
+        if CONFIG['thread']:
+            threading.Thread(
+                target=self.refresh_databases_start, args=(host, port)).start()
+            GLib.timeout_add(100, self.refresh_databases_end, host, port)
+        else:
+            self.refresh_databases_start(host, port)
+            self.refresh_databases_end(host, port)
 
     def refresh_databases_start(self, host, port):
         dbs = None
@@ -339,7 +346,8 @@ class DBListEditor(object):
             if index == -1:
                 index = 0
             self.database_combo.set_active(index)
-            self.database_entry.set_text(self.current_database
+            self.database_entry.set_text(
+                self.current_database
                 if self.current_database else '')
             if dbs:
                 self.database_combo.show()
@@ -403,7 +411,8 @@ class DBLogin(object):
     def __init__(self):
         # Fake windows to avoid warning about Dialog without transient
         self._window = Gtk.Window()
-        self.dialog = Gtk.Dialog(title="GNU Health - " + _('Login'), modal=True)
+        self.dialog = Gtk.Dialog(
+            title="GNU Health - " + _('Login'), modal=True)
         self.dialog.set_transient_for(self._window)
         self.dialog.set_icon(GNUHEALTH_ICON)
         self.dialog.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
@@ -412,15 +421,18 @@ class DBLogin(object):
 
         tooltips = common.Tooltips()
         button_cancel = Gtk.Button(label=_('_Cancel'), use_underline=True)
-        tooltips.set_tip(button_cancel,
-            _('Cancel connection to the  server'))
+        tooltips.set_tip(
+            button_cancel,
+            _('Cancel connection to the GNU Health server'))
         self.dialog.add_action_widget(button_cancel, Gtk.ResponseType.CANCEL)
         self.button_connect = Gtk.Button(
             label=_('C_onnect'), use_underline=True)
         self.button_connect.get_style_context().add_class(
             Gtk.STYLE_CLASS_SUGGESTED_ACTION)
         self.button_connect.set_can_default(True)
-        tooltips.set_tip(self.button_connect, _('Connect the GNU Health server'))
+        tooltips.set_tip(
+            self.button_connect, _('Connect the GNU Health server'))
+
         self.dialog.add_action_widget(self.button_connect, Gtk.ResponseType.OK)
         self.dialog.set_default_response(Gtk.ResponseType.OK)
         grid = Gtk.Grid(
@@ -438,7 +450,8 @@ class DBLogin(object):
         overlay = Gtk.Overlay()
         overlay.add(image)
         label = Gtk.Label(
-            label='<span color="white" background="#0090ad" ><b>%s</b></span>' % __version__,
+            '<span color="white" background="#0090ad">'
+            '<b>%s</b></span>' % __version__,
             use_markup=True)
         label.props.halign = Gtk.Align.END
         label.props.valign = Gtk.Align.START
@@ -474,8 +487,10 @@ class DBLogin(object):
             label=set_underline(_('Host:')),
             use_underline=True, halign=Gtk.Align.END)
         self.entry_host = Gtk.Entry(hexpand=True)
-        self.entry_host.connect_after('focus-out-event',
+        self.entry_host.connect_after(
+            'focus-out-event',
             self.clear_profile_combo)
+        self.entry_host.connect('focus-out-event', self.update_services)
         self.entry_host.set_activates_default(True)
         self.label_host.set_mnemonic_widget(self.entry_host)
         grid.attach(self.label_host, 0, 3, 1, 1)
@@ -484,7 +499,8 @@ class DBLogin(object):
             label=set_underline(_('Database:')),
             use_underline=True, halign=Gtk.Align.END)
         self.entry_database = Gtk.Entry(hexpand=True)
-        self.entry_database.connect_after('focus-out-event',
+        self.entry_database.connect_after(
+            'focus-out-event',
             self.clear_profile_combo)
         self.entry_database.set_activates_default(True)
         self.label_database.set_mnemonic_widget(self.entry_database)
@@ -500,24 +516,52 @@ class DBLogin(object):
         grid.attach(label_username, 0, 5, 1, 1)
 
         # Profile information
-        self.profile_cfg = os.path.join(get_config_dir(), 'profiles.cfg')
+        config_dir = get_config_dir()
+        self.profile_cfg = os.path.join(config_dir, 'profiles.cfg')
         self.profiles = configparser.ConfigParser()
-        if not os.path.exists(self.profile_cfg):
-            short_version = ''.join(__version__.split('.', 2)[:2])
-            name = 'federation.gnuhealth.org'
-            self.profiles.add_section(name)
-            self.profiles.set(name, 'host', name)
-            self.profiles.set(name, 'database', 'health%s' % short_version)
-            self.profiles.set(name, 'username', 'admin')
-        else:
-            try:
-                self.profiles.read(self.profile_cfg)
-            except configparser.ParsingError:
-                logger.error("Fail to parse profiles.cfg", exc_info=True)
+        series = '.'.join(__version__.split('.', 2)[:2])
+        current_demo = "federation.gnuhealth.org"
+        try:
+            self.profiles.read(self.profile_cfg)
+        except configparser.Error:
+            # reset self.profiles as parsing errors may leave wrong data in
+            # the parser
+            self.profiles = configparser.ConfigParser()
+            with tempfile.NamedTemporaryFile(
+                    delete=False, prefix='profiles_', suffix='.cfg',
+                    dir=config_dir) as temp_file:
+                temp_name = temp_file.name
+            shutil.copy(self.profile_cfg, temp_name)
+            logger.error(
+                f"Failed to parse {self.profiles_cfg}. "
+                f"A backup can be found at {temp_name}",
+                exc_info=True)
+
+        if not self.profiles.sections():
+            self.profiles.add_section(current_demo)
+            self.profiles.set(current_demo, 'host', current_demo)
+            self.profiles.set(current_demo, 'database', f'demo{series}')
+            self.profiles.set(current_demo, 'username', 'demo')
+        to_remove = []
         for section in self.profiles.sections():
-            active = all(self.profiles.has_option(section, option)
+            # host = self.profiles.get(section, 'host', fallback='')
+
+            active = all(
+                self.profiles.has_option(section, option)
                 for option in ('host', 'database'))
             self.profile_store.append([section, active])
+        for section in to_remove:
+            self.profiles.remove_section(section)
+        if to_remove and not self.profiles.has_section(current_demo):
+            self.profiles.add_section(current_demo)
+            self.profiles.set(current_demo, 'host', current_demo)
+            self.profiles.set(current_demo, 'database', f'demo{series}')
+            self.profiles.set(current_demo, 'username', 'demo')
+            self.profile_store.append([current_demo, True])
+
+        self.services = []
+        self.service_base = ''
+        self._services = {}
 
     def profile_manage(self, widget):
         def callback(profile_name):
@@ -530,8 +574,9 @@ class DBLogin(object):
                     self.profile_changed(self.combo_profile)
                     break
 
-        dia = DBListEditor(self.dialog, self.profile_store, self.profiles,
-            callback)
+        dia = DBListEditor(
+            self.dialog, self.profile_store, self.profiles, callback)
+
         active_profile = self.combo_profile.get_active()
         profile_name = None
         if active_profile != -1:
@@ -554,6 +599,7 @@ class DBLogin(object):
             self.entry_login.set_text(username)
         else:
             self.entry_login.set_text('')
+        self.update_services()
 
     def clear_profile_combo(self, *args):
         netloc = self.entry_host.get_text()
@@ -588,6 +634,33 @@ class DBLogin(object):
         self.entry_database.props.visible = visibility
         self.label_database.props.visible = visibility
 
+    def update_services(self, *args):
+        host = self.entry_host.get_text()
+        hostname = common.get_hostname(host)
+        port = common.get_port(host)
+        key = (hostname, port)
+        for response_id, service in enumerate(self.services, 1):
+            button = self.dialog.get_widget_for_response(response_id)
+            button.get_parent().remove(button)
+        self.services = []
+        if key in self._services:
+            self.services, self.service_base = self._services[key]
+        elif hostname and port:
+            self.service_base, self.services = rpc.authentication_services(
+                hostname, port)
+            self._services[key] = (self.services, self.service_base)
+        for response_id, (name, url) in enumerate(self.services, 1):
+            button = Gtk.Button(label=name)
+            self.dialog.add_action_widget(button, response_id)
+            button.show()
+
+        # Re-add connect button to be last
+        button_connect = self.dialog.get_widget_for_response(
+            Gtk.ResponseType.OK)
+        button_connect.get_parent().remove(button_connect)
+        self.dialog.add_action_widget(button_connect, Gtk.ResponseType.OK)
+        self.dialog.set_default_response(Gtk.ResponseType.OK)
+
     def run(self):
         profile_name = CONFIG['login.profile']
         can_use_profile = self.profiles.has_section(profile_name)
@@ -595,7 +668,7 @@ class DBLogin(object):
             for (configname, option) in [
                     ('login.host', 'host'),
                     ('login.db', 'database'),
-                    ]:
+            ]:
                 try:
                     value = self.profiles.get(profile_name, option)
                 except configparser.NoOptionError:
@@ -617,6 +690,7 @@ class DBLogin(object):
             self.entry_database.set_text(db)
             self.entry_login.set_text(CONFIG['login.login'])
             self.clear_profile_combo()
+        self.update_services()
         self.dialog.show_all()
 
         self.entry_login.grab_focus()
@@ -628,7 +702,7 @@ class DBLogin(object):
         response, result = None, ('', '', '', '')
         while not all(result):
             response = self.dialog.run()
-            if response != Gtk.ResponseType.OK:
+            if response != Gtk.ResponseType.OK and response <= 0:
                 break
             self.clear_profile_combo()
             active_profile = self.combo_profile.get_active()
@@ -642,24 +716,31 @@ class DBLogin(object):
             test = DBListEditor.test_server_version(hostname, port)
             if not test:
                 if test is False:
-                    common.warning('',
-                        _('Incompatible version of the server.'),
+                    common.warning(
+                        '', _('Incompatible version of the server.'),
                         parent=self.dialog)
                 else:
-                    common.warning('',
-                        _('Could not connect to the server.'),
+                    common.warning(
+                        '', _('Could not connect to the server.'),
                         parent=self.dialog)
                 continue
             database = self.entry_database.get_text()
-            login = self.entry_login.get_text()
             CONFIG['login.profile'] = profile
             CONFIG['login.host'] = host
             CONFIG['login.db'] = database
             CONFIG['login.expanded'] = self.expander.props.expanded
-            CONFIG['login.login'] = login
-            result = (
-                hostname, port, database, self.entry_login.get_text())
+            if response == Gtk.ResponseType.OK:
+                authentication = self.entry_login.get_text()
+                CONFIG['login.login'] = authentication
+                CONFIG['login.service'] = ''
+            else:
+                authentication = self.services[response - 1][1]
+                authentication = (
+                    self.service_base + database + authentication)
+                CONFIG['login.login'] = ''
+                CONFIG['login.service'] = authentication
+            result = (hostname, port, database, authentication)
 
         self.dialog.destroy()
         self._window.destroy()
-        return response == Gtk.ResponseType.OK
+        return response == Gtk.ResponseType.OK or response > 0
