@@ -5,23 +5,26 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2023 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from gnuhealth.signal_event import SignalEvent
+import logging
+
 import gnuhealth.common as common
-from gnuhealth.pyson import PYSONDecoder
-from . import field as fields
-from gnuhealth.common import RPCExecute, RPCException
+from gnuhealth.common import RPCException, RPCExecute
 from gnuhealth.config import CONFIG
+from gnuhealth.pyson import PYSONDecoder
+
+from . import field as fields
+
+logger = logging.getLogger(__name__)
 
 
-class Record(SignalEvent):
+class Record:
 
     id = -1
 
@@ -53,10 +56,15 @@ class Record(SignalEvent):
         self.destroyed = False
 
     def __getitem__(self, name):
+        self.load(name)
+        if name != '*':
+            return self.group.fields[name]
+
+    def load(self, name, process_exception=True):
         if not self.destroyed and self.id >= 0 and name not in self._loaded:
             id2record = {
                 self.id: self,
-                }
+            }
             if name == '*':
                 loading = 'eager'
                 views = set()
@@ -75,36 +83,40 @@ class Record(SignalEvent):
 
             if loading == 'eager':
                 fields = ((fname, field)
-                    for fname, field in self.group.fields.items()
-                    if field.attrs.get('loading', 'eager') == 'eager')
+                          for fname, field in self.group.fields.items()
+                          if field.attrs.get('loading', 'eager') == 'eager')
             else:
                 fields = self.group.fields.items()
 
             fnames = [fname for fname, field in fields
-                if fname not in self._loaded
-                and (not views or (views & field.views))]
-            fnames.extend(('%s.rec_name' % fname for fname in fnames[:]
-                    if self.group.fields[fname].attrs['type']
-                    in ('many2one', 'one2one', 'reference')))
+                      if fname not in self._loaded
+                      and (not views or (views & field.views))]
+            for fname in list(fnames):
+                f_attrs = self.group.fields[fname].attrs
+                if f_attrs['type'] in {'many2one', 'one2one', 'reference'}:
+                    fnames.append('%s.rec_name' % fname)
+                elif (f_attrs['type'] in {'selection', 'multiselection'}
+                        and f_attrs.get('loading', 'lazy') == 'eager'):
+                    fnames.append('%s:string' % fname)
             if 'rec_name' not in fnames:
                 fnames.append('rec_name')
             fnames.extend(['_timestamp', '_write', '_delete'])
 
             record_context = self.get_context()
             if loading == 'eager':
-                limit = int(CONFIG['client.limit'] / len(fnames))
+                limit = CONFIG['client.limit'] // min(len(fnames), 10)
 
                 def filter_group(record):
                     return (not record.destroyed
-                        and record.id >= 0
-                        and name not in record._loaded)
+                            and record.id >= 0
+                            and name not in record._loaded)
 
                 def filter_parent_group(record):
                     return (filter_group(record)
-                        and record.id not in id2record
-                        and ((record.group == self.group)
+                            and record.id not in id2record
+                            and ((record.group == self.group)
                             # Don't compute context for same group
-                            or (record.get_context() == record_context)))
+                                 or (record.get_context() == record_context)))
 
                 if self.parent and self.parent.model_name == self.model_name:
                     group = sum(self.parent.group.children, [])
@@ -116,7 +128,8 @@ class Record(SignalEvent):
                     idx = group.index(self)
                     length = len(group)
                     n = 1
-                    while len(id2record) < limit and (idx - n >= 0
+                    while len(id2record) < limit and (
+                        idx - n >= 0
                             or idx + n < length) and n < 2 * limit:
                         if idx - n >= 0:
                             record = group[idx - n]
@@ -130,36 +143,44 @@ class Record(SignalEvent):
 
             ctx = record_context.copy()
             ctx.update(dict(('%s.%s' % (self.model_name, fname), 'size')
-                    for fname, field in self.group.fields.items()
-                    if field.attrs['type'] == 'binary' and fname in fnames))
-            exception = False
+                            for fname, field in self.group.fields.items()
+                            if field.attrs['type'] == 'binary'
+                            and fname in fnames))
             try:
-                values = RPCExecute('model', self.model_name, 'read',
-                    list(id2record.keys()), fnames, context=ctx)
+                values = RPCExecute(
+                    'model', self.model_name, 'read',
+                    list(id2record.keys()), fnames, context=ctx,
+                    process_exception=process_exception)
             except RPCException:
-                values = [{'id': x} for x in id2record]
-                default_values = dict((f, None) for f in fnames)
-                for value in values:
-                    value.update(default_values)
-                self.exception = exception = True
+                for record in id2record.values():
+                    record.exception = True
+                if process_exception:
+                    values = [{'id': x} for x in id2record]
+                    default_values = {f: None for f in fnames if f != 'id'}
+                    for value in values:
+                        value.update(default_values)
+                else:
+                    raise
             id2value = dict((value['id'], value) for value in values)
             for id, record in id2record.items():
-                if not record.exception:
-                    record.exception = exception
                 value = id2value.get(id)
                 if record and not record.destroyed and value:
                     for key in record.modified_fields:
                         value.pop(key, None)
-                    record.set(value, signal=False)
-        if name != '*':
-            return self.group.fields[name]
+                    record.set(value, modified=False)
 
     def __repr__(self):
         return '<Record %s@%s at %s>' % (self.id, self.model_name, id(self))
 
     @property
     def modified(self):
-        return bool(self.modified_fields)
+        if self.modified_fields:
+            logger.info(
+                "Modified fields %s of %s",
+                list(self.modified_fields.keys()), self)
+            return True
+        else:
+            return False
 
     @property
     def parent(self):
@@ -185,10 +206,6 @@ class Record(SignalEvent):
             parent = parent.parent
         return i
 
-    def set_modified(self, value):
-        if value:
-            self.signal('record-modified')
-
     def children_group(self, field_name):
         if not field_name:
             return []
@@ -245,9 +262,10 @@ class Record(SignalEvent):
 
     def get_readonly(self):
         return (self.deleted
-            or self.removed
-            or self.exception
-            or not self._write)
+                or self.removed
+                or self.exception
+                or self.group.readonly
+                or not self._write)
 
     readonly = property(get_readonly)
 
@@ -281,7 +299,7 @@ class Record(SignalEvent):
         for name, field in self.group.fields.items():
             if (field.attrs.get('readonly')
                     and not (isinstance(field, fields.O2MField)
-                        and not isinstance(field, fields.M2MField))):
+                             and not isinstance(field, fields.M2MField))):
                 continue
             if field.name not in self.modified_fields and self.id >= 0:
                 continue
@@ -307,7 +325,7 @@ class Record(SignalEvent):
                 continue
             if (self.id >= 0
                     and (name not in self._loaded
-                        or name not in self.modified_fields)):
+                         or name not in self.modified_fields)):
                 continue
             value[name] = field.get_on_change_value(self)
         value['id'] = self.id
@@ -315,10 +333,12 @@ class Record(SignalEvent):
 
     def cancel(self):
         self._loaded.clear()
+        self.value = {}
         self.modified_fields.clear()
         self._timestamp = None
         self.button_clicks.clear()
         self.links_counts.clear()
+        self.exception = False
 
     def get_timestamp(self):
         result = {self.model_name + ',' + str(self.id): self._timestamp}
@@ -333,7 +353,7 @@ class Record(SignalEvent):
         values = self._get_on_change_args(['id'] + list(self.modified_fields))
         try:
             RPCExecute('model', self.model_name, 'pre_validate', values,
-                context=self.get_context())
+                       context=self.get_context())
         except RPCException:
             return False
         return True
@@ -344,7 +364,7 @@ class Record(SignalEvent):
             if self.id < 0:
                 try:
                     res, = RPCExecute('model', self.model_name, 'create',
-                        [value], context=self.get_context())
+                                      [value], context=self.get_context())
                 except RPCException:
                     return False
                 old_id = self.id
@@ -357,7 +377,7 @@ class Record(SignalEvent):
                     context['_timestamp'] = self.get_timestamp()
                     try:
                         RPCExecute('model', self.model_name, 'write',
-                            [self.id], value, context=context)
+                                   [self.id], value, context=context)
                     except RPCException:
                         return False
             self.cancel()
@@ -370,12 +390,15 @@ class Record(SignalEvent):
             self.parent.save(force_reload=force_reload)
         return self.id
 
-    def default_get(self, rec_name=None):
+    def default_get(self, defaults=None):
         if len(self.group.fields):
             context = self.get_context()
-            context.setdefault('default_rec_name', rec_name)
+            if defaults is not None:
+                for name, value in defaults.items():
+                    context.setdefault(f'default_{name}', value)
             try:
-                vals = RPCExecute('model', self.model_name, 'default_get',
+                vals = RPCExecute(
+                    'model', self.model_name, 'default_get',
                     list(self.group.fields.keys()), context=context)
             except RPCException:
                 return
@@ -393,7 +416,8 @@ class Record(SignalEvent):
 
     def rec_name(self):
         try:
-            return RPCExecute('model', self.model_name, 'read', [self.id],
+            return RPCExecute(
+                'model', self.model_name, 'read', [self.id],
                 ['rec_name'], context=self.get_context())[0]['rec_name']
         except RPCException:
             return ''
@@ -431,17 +455,22 @@ class Record(SignalEvent):
         else:
             return self.group.local_context
 
-    def set_default(self, val, signal=True, validate=True):
+    def set_default(self, val, modified=True, validate=True):
         fieldnames = []
         for fieldname, value in list(val.items()):
+            if fieldname in {'_write', '_delete', '_timestamp'}:
+                setattr(self, fieldname, value)
+                continue
             if fieldname not in self.group.fields:
                 continue
             if fieldname == self.group.exclude_field:
                 continue
-            if isinstance(self.group.fields[fieldname], (fields.M2OField,
-                        fields.ReferenceField)):
+            if isinstance(
+                self.group.fields[fieldname],
+                    (fields.M2OField, fields.ReferenceField)):
                 related = fieldname + '.'
                 self.value[related] = val.get(related) or {}
+
             self.group.fields[fieldname].set_default(self, value)
             self._loaded.add(fieldname)
             fieldnames.append(fieldname)
@@ -449,10 +478,10 @@ class Record(SignalEvent):
         self.on_change_with(fieldnames)
         if validate:
             self.validate(softvalidation=True)
-        if signal:
-            self.signal('record-changed')
+        if modified:
+            self.set_modified()
 
-    def set(self, val, signal=True, validate=True):
+    def set(self, val, modified=True, validate=True):
         later = {}
         fieldnames = []
         for fieldname, value in val.items():
@@ -468,32 +497,43 @@ class Record(SignalEvent):
                 if fieldname == 'rec_name':
                     self.value['rec_name'] = value
                 continue
-            if isinstance(self.group.fields[fieldname], fields.O2MField):
+            field = self.group.fields[fieldname]
+            if isinstance(field, fields.O2MField):
                 later[fieldname] = value
                 continue
-            if isinstance(self.group.fields[fieldname], (fields.M2OField,
-                        fields.ReferenceField)):
+            if isinstance(field, (fields.M2OField, fields.ReferenceField)):
                 related = fieldname + '.'
                 self.value[related] = val.get(related) or {}
+            elif isinstance(field, (
+                    fields.SelectionField,
+                    fields.MultiSelectionField)):
+                related = fieldname + ':string'
+                if fieldname + ':string' in val:
+                    self.value[related] = val[related]
+                else:
+                    self.value.pop(related, None)
             self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
             fieldnames.append(fieldname)
         for fieldname, value in later.items():
             self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
+            fieldnames.append(fieldname)
         if validate:
             self.validate(fieldnames, softvalidation=True)
-        if signal:
-            self.signal('record-changed')
+        if modified:
+            self.set_modified()
 
     def set_on_change(self, values):
         for fieldname, value in list(values.items()):
             if fieldname not in self.group.fields:
                 continue
-            if isinstance(self.group.fields[fieldname], (fields.M2OField,
-                        fields.ReferenceField)):
+            if isinstance(
+                self.group.fields[fieldname],
+                    (fields.M2OField, fields.ReferenceField)):
                 related = fieldname + '.'
                 self.value[related] = values.get(related) or {}
+
             # Load fieldname before setting value
             self[fieldname].set_on_change(self, value)
 
@@ -509,13 +549,13 @@ class Record(SignalEvent):
 
     def reset(self, value):
         self.cancel()
-        self.set(value, signal=False)
+        self.set(value, modified=False)
 
         if self.parent:
             self.parent.on_change([self.group.child_name])
             self.parent.on_change_with([self.group.child_name])
 
-        self.signal('record-changed')
+        self.set_modified()
 
     def expr_eval(self, expr):
         if not isinstance(expr, str):
@@ -559,20 +599,34 @@ class Record(SignalEvent):
 
         if values:
             try:
-                if len(fieldnames) == 1:
-                    fieldname, = fieldnames
+                if len(fieldnames) == 1 or 'id' not in values:
                     changes = []
-                    changes.append(RPCExecute(
-                            'model', self.model_name, 'on_change_' + fieldname,
+                    for fieldname in fieldnames:
+                        changes.append(RPCExecute(
+                            'model', self.model_name,
+                            'on_change_' + fieldname,
                             values, context=self.get_context()))
                 else:
-                    changes = RPCExecute(
+                    changes = [RPCExecute(
                         'model', self.model_name, 'on_change',
-                        values, fieldnames, context=self.get_context())
+                        values, fieldnames, context=self.get_context())]
             except RPCException:
-                return
-            for change in changes:
-                self.set_on_change(change)
+                pass
+            else:
+                for change in changes:
+                    self.set_on_change(change)
+
+        notification_fields = common.MODELNOTIFICATION.get(self.model_name)
+        if set(fieldnames) & set(notification_fields):
+            values = self._get_on_change_args(notification_fields)
+            try:
+                notifications = RPCExecute(
+                    'model', self.model_name, 'on_change_notify', values,
+                    context=self.get_context())
+            except RPCException:
+                pass
+            else:
+                self.group.record_notify(notifications)
 
     def on_change_with(self, field_names):
         field_names = set(field_names)
@@ -581,7 +635,7 @@ class Record(SignalEvent):
         later = set()
         for fieldname in self.group.fields:
             on_change_with = self.group.fields[fieldname].attrs.get(
-                    'on_change_with')
+                'on_change_with')
             if not on_change_with:
                 continue
             if not field_names & set(on_change_with):
@@ -590,38 +644,51 @@ class Record(SignalEvent):
                 later.add(fieldname)
                 continue
             fieldnames.add(fieldname)
-            values.update(self._get_on_change_args(on_change_with))
-            if isinstance(self.group.fields[fieldname], (fields.M2OField,
-                        fields.ReferenceField)):
+            values.update(
+                self._get_on_change_args(on_change_with + [fieldname]))
+            if isinstance(
+                self.group.fields[fieldname],
+                    (fields.M2OField, fields.ReferenceField)):
                 self.value.pop(fieldname + '.', None)
+
         if fieldnames:
             try:
-                if len(fieldnames) == 1:
-                    fieldname, = fieldnames
-                    result = {}
-                    result[fieldname] = RPCExecute(
-                        'model', self.model_name,
-                        'on_change_with_' + fieldname,
-                        values, context=self.get_context())
+                if len(fieldnames) == 1 or 'id' not in values:
+                    changed = {}
+                    for fieldname in fieldnames:
+                        changed.update(RPCExecute(
+                            'model', self.model_name,
+                            'on_change_with_' + fieldname,
+                            values, context=self.get_context()))
                 else:
-                    result = RPCExecute(
+                    changed = RPCExecute(
                         'model', self.model_name, 'on_change_with',
                         values, list(fieldnames), context=self.get_context())
             except RPCException:
                 return
-            self.set_on_change(result)
-        for fieldname in later:
-            on_change_with = self.group.fields[fieldname].attrs.get(
+            self.set_on_change(changed)
+        if later:
+            values = {}
+            for fieldname in later:
+                on_change_with = self.group.fields[fieldname].attrs.get(
                     'on_change_with')
-            values = self._get_on_change_args(on_change_with)
+                values.update(
+                    self._get_on_change_args(on_change_with + [fieldname]))
             try:
-                result = RPCExecute('model', self.model_name,
-                    'on_change_with_' + fieldname, values,
-                    context=self.get_context())
+                if len(later) == 1 or 'id' not in values:
+                    changed = {}
+                    for fieldname in later:
+                        changed.update(RPCExecute(
+                            'model', self.model_name,
+                            'on_change_with_' + fieldname,
+                            values, context=self.get_context()))
+                else:
+                    changed = RPCExecute(
+                        'model', self.model_name, 'on_change_with',
+                        values, list(later), context=self.get_context())
             except RPCException:
                 return
-            # Load fieldname before setting value
-            self[fieldname].set_on_change(self, result)
+            self.set_on_change(changed)
 
     def autocomplete_with(self, field_name):
         for fieldname, fieldinfo in self.group.fields.items():
@@ -643,6 +710,16 @@ class Record(SignalEvent):
             res = []
         self.autocompletion[fieldname] = res
 
+    def on_scan_code(self, code, depends):
+        depends = self.expr_eval(depends)
+        values = self._get_on_change_args(depends)
+        changes = RPCExecute(
+            'model', self.model_name, 'on_scan_code', values, code,
+            context=self.get_context(), process_exception=False)
+        self.set_on_change(changes)
+        self.set_modified()
+        return bool(changes)
+
     def set_field_context(self):
         from .group import Group
         for name, field in self.group.fields.items():
@@ -671,15 +748,19 @@ class Record(SignalEvent):
             return clicks
         try:
             clicks = RPCExecute('model', 'ir.model.button.click',
-                'get_click', self.model_name, name, self.id)
+                                'get_click', self.model_name, name, self.id)
             self.button_clicks[name] = clicks
         except RPCException:
             return
         return clicks
 
+    def set_modified(self, field=None):
+        if field:
+            self.modified_fields.setdefault(field)
+        self.group.record_modified()
+
     def destroy(self):
         for v in self.value.values():
             if hasattr(v, 'destroy'):
                 v.destroy()
-        super(Record, self).destroy()
         self.destroyed = True
