@@ -5,12 +5,11 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2022 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import gettext
@@ -18,13 +17,14 @@ import gettext
 from gi.repository import Gdk, Gtk
 
 import gnuhealth.common as common
-from .widget import Widget
 from gnuhealth.common.completion import get_completion, update_completion
 from gnuhealth.common.domain_parser import quote
 from gnuhealth.common.underline import set_underline
 from gnuhealth.gui.window.view_form.screen import Screen
-from gnuhealth.gui.window.win_search import WinSearch
 from gnuhealth.gui.window.win_form import WinForm
+from gnuhealth.gui.window.win_search import WinSearch
+
+from .widget import Widget
 
 _ = gettext.gettext
 
@@ -65,12 +65,12 @@ class Many2Many(Widget):
 
         if int(self.attrs.get('completion', 1)):
             self.wid_completion = get_completion(
-                create=self.attrs.get('create', True)
-                and common.MODELACCESS[self.attrs['relation']]['create'])
+                search=self.read_access,
+                create=self.create_access)
             self.wid_completion.connect('match-selected',
-                self._completion_match_selected)
+                                        self._completion_match_selected)
             self.wid_completion.connect('action-activated',
-                self._completion_action_activated)
+                                        self._completion_action_activated)
             self.wid_text.set_completion(self.wid_completion)
             self.wid_text.connect('changed', self._update_completion)
         else:
@@ -80,15 +80,18 @@ class Many2Many(Widget):
         tooltips.set_tip(self.but_add, _('Add existing record'))
         self.but_add.connect('clicked', self._sig_add)
         self.but_add.add(common.IconFactory.get_image(
-                'gnuhealth-add', Gtk.IconSize.SMALL_TOOLBAR))
+            'gnuhealth-add', Gtk.IconSize.SMALL_TOOLBAR))
         self.but_add.set_relief(Gtk.ReliefStyle.NONE)
         hbox.pack_start(self.but_add, expand=False, fill=False, padding=0)
 
+        self.label = Gtk.Label(label='(_/0)')
+        hbox.pack_start(self.label, expand=False, fill=False, padding=0)
+
         self.but_remove = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_remove, _('Remove selected record'))
         self.but_remove.connect('clicked', self._sig_remove)
         self.but_remove.add(common.IconFactory.get_image(
-                'gnuhealth-remove', Gtk.IconSize.SMALL_TOOLBAR))
+            'gnuhealth-remove', Gtk.IconSize.SMALL_TOOLBAR))
         self.but_remove.set_relief(Gtk.ReliefStyle.NONE)
         hbox.pack_start(self.but_remove, expand=False, fill=False, padding=0)
 
@@ -99,14 +102,22 @@ class Many2Many(Widget):
         frame.set_shadow_type(Gtk.ShadowType.OUT)
         vbox.pack_start(frame, expand=False, fill=True, padding=0)
 
-        self.screen = Screen(attrs['relation'],
+        model = attrs['relation']
+        breadcrumb = list(self.view.screen.breadcrumb)
+        breadcrumb.append(
+            attrs.get('string') or common.MODELNAME.get(model))
+        self.screen = Screen(
+            model,
             view_ids=attrs.get('view_ids', '').split(','),
             mode=['tree'], views_preload=attrs.get('views', {}),
             order=attrs.get('order'),
             row_activate=self._on_activate,
             readonly=True,
-            limit=None)
-        self.screen.signal_connect(self, 'record-message', self._sig_label)
+            limit=None,
+            context=self.view.screen.context,
+            breadcrumb=breadcrumb)
+
+        self.screen.windows.append(self)
 
         vbox.pack_start(self.screen.widget, expand=True, fill=True, padding=0)
 
@@ -148,6 +159,21 @@ class Many2Many(Widget):
         self.wid_text.disconnect_by_func(self._focus_out)
         self.screen.destroy()
 
+    def get_access(self, type_):
+        model = self.attrs['relation']
+        if model:
+            return common.MODELACCESS[model][type_]
+        else:
+            return True
+
+    @property
+    def read_access(self):
+        return self.get_access('read')
+
+    @property
+    def create_access(self):
+        return int(self.attrs.get('create', 1)) and self.get_access('create')
+
     def _sig_add(self, *args):
         if not self.focus_out:
             return
@@ -166,23 +192,16 @@ class Many2Many(Widget):
             if result:
                 ids = [x[0] for x in result]
                 self.screen.load(ids, modified=True)
-                self.screen.display(res_id=ids[0])
             self.screen.set_cursor()
             self.wid_text.set_text('')
         win = WinSearch(self.attrs['relation'], callback, sel_multi=True,
-            context=context, domain=domain, order=order,
-            view_ids=self.attrs.get('view_ids', '').split(','),
-            views_preload=self.attrs.get('views', {}),
-            new=self.attrs.get('create', True),
-            title=self.attrs.get('string'))
+                        context=context, domain=domain, order=order,
+                        view_ids=self.attrs.get('view_ids', '').split(','),
+                        views_preload=self.attrs.get('views', {}),
+                        new=self.create_access,
+                        title=self.attrs.get('string'))
         win.screen.search_filter(quote(value))
-        if len(win.screen.group) == 1:
-            callback([
-                    (r.id, r.value.get('rec_name', ''))
-                    for r in win.screen.group])
-            win.destroy()
-        else:
-            win.show()
+        win.show()
 
     def _sig_remove(self, *args):
         self.screen.remove(remove=True)
@@ -203,9 +222,9 @@ class Many2Many(Widget):
         breadcrumb.append(
             self.attrs.get('string') or common.MODELNAME.get(model))
         return Screen(model, domain=domain,
-            view_ids=view_ids,
-            mode=['form'], views_preload=self.attrs.get('views', {}),
-            context=context, breadcrumb=breadcrumb)
+                      view_ids=view_ids,
+                      mode=['form'], views_preload=self.attrs.get('views', {}),
+                      context=context, breadcrumb=breadcrumb)
 
     def _sig_edit(self):
         if not self.screen.current_record:
@@ -214,18 +233,24 @@ class Many2Many(Widget):
         # save of the record will trigger the save of the parent
         screen = self._get_screen_form()
         screen.load([self.screen.current_record.id])
+        screen.current_record = screen.group.get(self.screen.current_record.id)
 
         def callback(result):
             if result:
                 screen.current_record.save()
+                added = 'id' in self.screen.current_record.modified_fields
                 # Force a reload on next display
                 self.screen.current_record.cancel()
+                if added:
+                    self.screen.current_record.modified_fields.setdefault('id')
                 # Force a display to clear the CellCache
                 self.screen.display()
         WinForm(screen, callback)
 
-    def _sig_new(self):
+    def _sig_new(self, defaults=None):
         screen = self._get_screen_form()
+        defaults = defaults.copy() if defaults is not None else {}
+        defaults['rec_name'] = self.wid_text.get_text()
 
         def callback(result):
             self.focus_out = True
@@ -236,8 +261,8 @@ class Many2Many(Widget):
             self.wid_text.grab_focus()
 
         self.focus_out = False
-        WinForm(screen, callback, new=True, save_current=True,
-            rec_name=self.wid_text.get_text())
+        WinForm(
+            screen, callback, new=True, save_current=True, defaults=defaults)
 
     def _readonly_set(self, value):
         self._readonly = value
@@ -259,19 +284,25 @@ class Many2Many(Widget):
             field_size = self.record.expr_eval(self.attrs.get('size'))
             m2m_size = len(self.field.get_eval(self.record))
             size_limit = (field_size is not None
-                and m2m_size >= field_size >= 0)
+                          and m2m_size >= field_size >= 0)
         else:
             size_limit = False
 
         self.but_add.set_sensitive(bool(
-                not self._readonly
-                and not size_limit))
+            not self._readonly
+            and not size_limit))
         self.but_remove.set_sensitive(bool(
-                not self._readonly
-                and self._position))
+            not self._readonly
+            and self._position))
 
-    def _sig_label(self, screen, signal_data):
-        self._position = signal_data[0]
+    def record_message(self, position, size, *args):
+        self._position = position
+        name = str(position) if position else '_'
+        selected = len(self.screen.selected_records)
+        if selected > 1:
+            name += '#%i' % selected
+        name = '(%s/%s)' % (name, common.humanize(size))
+        self.label.set_text(name)
         self._set_button_sensitive()
 
     def display(self):
@@ -293,14 +324,17 @@ class Many2Many(Widget):
         return True
 
     def _completion_match_selected(self, completion, model, iter_):
-        record_id, = model.get(iter_, 1)
-        self.screen.load([record_id], modified=True)
-        self.wid_text.set_text('')
-        self.wid_text.grab_focus()
+        record_id, defaults = model.get(iter_, 1, 2)
+        if record_id is not None:
+            self.screen.load([record_id], modified=True)
+            self.wid_text.set_text('')
+            self.wid_text.grab_focus()
 
-        completion_model = self.wid_completion.get_model()
-        completion_model.clear()
-        completion_model.search_text = self.wid_text.get_text()
+            completion_model = self.wid_completion.get_model()
+            completion_model.clear()
+            completion_model.search_text = self.wid_text.get_text()
+        else:
+            self._sig_new(defaults)
         return True
 
     def _update_completion(self, widget):
