@@ -5,12 +5,11 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2023 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import gettext
@@ -18,8 +17,7 @@ import gettext
 from gi.repository import Gdk, GLib, Gtk
 
 import gnuhealth.common as common
-from gnuhealth.common import RPCExecute, RPCException
-from gnuhealth.common import GNUHEALTH_ICON
+from gnuhealth.common import GNUHEALTH_ICON, RPCException, RPCExecute
 from gnuhealth.common.underline import set_underline
 from gnuhealth.common.widget_style import widget_class
 from gnuhealth.gui import Main
@@ -30,7 +28,7 @@ _ = gettext.gettext
 
 class Widget(object):
     expand = False
-    default_width_chars = 25
+    default_width_chars = 8
 
     def __init__(self, view, attrs):
         super(Widget, self).__init__()
@@ -91,8 +89,8 @@ class Widget(object):
         def send(value):
             if not self.widget.props.window:
                 return
-            if self.record and self.get_value() == value:
-                self.record.signal('record-modified')
+            if self.record and self.get_value() == value and self.modified:
+                self.view.screen.record_modified(display=False)
 
         def get_value():
             if not self.widget.props.window:
@@ -137,7 +135,10 @@ class Widget(object):
         invalid = states.get('invalid', False)
         widget_class(self._invalid_widget, 'invalid', not readonly and invalid)
         self.invisible_set(self.attrs.get(
-                'invisible', states.get('invisible', False)))
+            'invisible', states.get('invisible', False)))
+
+    def get_value(self):
+        pass
 
     def set_value(self):
         pass
@@ -163,7 +164,8 @@ class TranslateDialog(NoModal):
         cancel_button = self.win.add_button(
             set_underline(_("Cancel")), Gtk.ResponseType.CANCEL)
         cancel_button.set_image(
-            common.IconFactory.get_image('gnuhealth-cancel', Gtk.IconSize.BUTTON))
+            common.IconFactory.get_image(
+                'gnuhealth-cancel', Gtk.IconSize.BUTTON))
         cancel_button.set_always_show_image(True)
         ok_button = self.win.add_button(
             set_underline(_("OK")), Gtk.ResponseType.OK)
@@ -184,23 +186,26 @@ class TranslateDialog(NoModal):
                 label=label,
                 halign=Gtk.Align.END,
                 valign=(Gtk.Align.START if self.widget.expand
-                    else Gtk.Align.FILL))
+                        else Gtk.Align.FILL))
             grid.attach(label, 0, i, 1, 1)
 
             context = dict(
                 language=language['code'],
                 fuzzy_translation=False,
-                )
+            )
             try:
                 value = RPCExecute('model', self.widget.record.model_name,
-                    'read', [self.widget.record.id], [self.widget.field_name],
-                    context={'language': language['code']}
-                    )[0][self.widget.field_name]
+                                   'read', [
+                                       self.widget.record.id], [
+                                       self.widget.field_name],
+                                   context={'language': language['code']}
+                                   )[0][self.widget.field_name]
             except RPCException:
                 return
             context['fuzzy_translation'] = True
             try:
-                fuzzy_value = RPCExecute('model',
+                fuzzy_value = RPCExecute(
+                    'model',
                     self.widget.record.model_name, 'read',
                     [self.widget.record.id], [self.widget.field_name],
                     context=context)[0][self.widget.field_name]
@@ -215,17 +220,15 @@ class TranslateDialog(NoModal):
             widget.set_vexpand(self.widget.expand)
             widget.set_hexpand(True)
             grid.attach(widget, 1, i, 1, 1)
-            editing = Gtk.CheckButton()
+            editing = Gtk.ToggleButton(label=_("Edit"))
             editing.connect('toggled', self.editing_toggled, widget)
             editing.props.sensitive = not readonly
-            tooltips.set_tip(editing, _('Edit'))
             grid.attach(editing, 2, i, 1, 1)
-            fuzzy = Gtk.CheckButton()
-            fuzzy.set_active(value != fuzzy_value)
-            fuzzy.props.sensitive = False
-            tooltips.set_tip(fuzzy, _('Fuzzy'))
-            grid.attach(fuzzy, 4, i, 1, 1)
-            self.widgets[language['code']] = (widget, editing, fuzzy)
+            if value != fuzzy_value:
+                fuzzy = Gtk.Label(_("Fuzzy"))
+                widget_class(fuzzy, 'warning', True)
+                grid.attach(fuzzy, 4, i, 1, 1)
+            self.widgets[language['code']] = (widget, editing)
 
         tooltips.enable()
         vbox = Gtk.VBox()
@@ -247,24 +250,24 @@ class TranslateDialog(NoModal):
 
     def editing_toggled(self, editing, widget):
         self.widget.translate_widget_set_readonly(widget,
-            not editing.get_active())
+                                                  not editing.get_active())
 
     def response(self, win, response):
         if response == Gtk.ResponseType.OK:
             for code, widget in self.widgets.items():
-                widget, editing, fuzzy = widget
+                widget, editing = widget
                 if not editing.get_active():
                     continue
                 value = self.widget.translate_widget_get(widget)
                 context = dict(
                     language=code,
                     fuzzy_translation=False,
-                    )
+                )
                 try:
                     RPCExecute('model', self.widget.record.model_name, 'write',
-                        [self.widget.record.id], {
-                            self.widget.field_name: value,
-                            }, context=context)
+                               [self.widget.record.id], {
+                                   self.widget.field_name: value,
+                               }, context=context)
                 except RPCException:
                     pass
             self.widget.record.cancel()
@@ -287,7 +290,7 @@ class TranslateMixin:
     def translate_button(self):
         button = Gtk.Button()
         button.set_image(common.IconFactory.get_image(
-                'gnuhealth-translate', Gtk.IconSize.SMALL_TOOLBAR))
+            'gnuhealth-translate', Gtk.IconSize.SMALL_TOOLBAR))
         button.set_relief(Gtk.ReliefStyle.NONE)
         button.connect('clicked', self.translate)
         return button
@@ -303,8 +306,8 @@ class TranslateMixin:
 
         try:
             lang_ids = RPCExecute('model', 'ir.lang', 'search', [
-                    ('translatable', '=', True),
-                    ])
+                ('translatable', '=', True),
+            ])
         except RPCException:
             return
 
@@ -313,7 +316,7 @@ class TranslateMixin:
             return
         try:
             languages = RPCExecute('model', 'ir.lang', 'read', lang_ids,
-                ['code', 'name'])
+                                   ['code', 'name'])
         except RPCException:
             return
 
