@@ -5,39 +5,39 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2023 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import sys
+import gettext
 import json
 import locale
-import gettext
+import sys
 from functools import wraps
 
 from gi.repository import Gdk, GLib, GObject, Gtk
+# Import Simon Feltman generictreemodel
 from .generictreemodel import GenericTreeModel
 
-# Import Simon Feltman generictreemodel 
-from .generictreemodel import GenericTreeModel
-
-from gnuhealth.config import CONFIG
-from gnuhealth.common.cellrendererbutton import CellRendererButton
-from gnuhealth.gui.window import Window
-from gnuhealth.common.popup_menu import populate
-from gnuhealth.common import RPCExecute, RPCException, node_attributes, Tooltips
-from gnuhealth.common import domain_inversion, simplify, unique_value
-from gnuhealth.pyson import PYSONDecoder
 import gnuhealth.common as common
+from gnuhealth.common import (
+    RPCException, RPCExecute, Tooltips, domain_inversion, node_attributes,
+    simplify, unique_value)
+from gnuhealth.common.cellrendererbutton import CellRendererButton
+from gnuhealth.common.popup_menu import populate, popup
+from gnuhealth.config import CONFIG
+from gnuhealth.gui.window import Window
+from gnuhealth.pyson import PYSONDecoder
+
 from . import View, XMLViewParser
 from .list_gtk.editabletree import EditableTreeView, TreeView
-from .list_gtk.widget import (Affix, Char, Text, Int, Boolean, URL, Date,
-    Time, Float, TimeDelta, Binary, M2O, O2O, O2M, M2M, Selection,
-    MultiSelection, Reference, Dict, ProgressBar, Button, Image)
+from .list_gtk.widget import (
+    M2M, M2O, O2M, O2O, URL, Affix, Binary, Boolean, Button, Char, Date, Dict,
+    Float, Image, Int, MultiSelection, ProgressBar, Reference, Selection, Text,
+    Time, TimeDelta)
 
 _ = gettext.gettext
 
@@ -85,7 +85,7 @@ class AdaptModelGroup(GenericTreeModel):
     def added(self, group, record):
         if (group is self.group
                 and (record.group is self.group
-                    or record.group.child_name == self.children_field)):
+                     or record.group.child_name == self.children_field)):
             path = record.get_index_path(self.group)
             iter_ = self.get_iter(path)
             self.row_inserted(path, iter_)
@@ -100,7 +100,7 @@ class AdaptModelGroup(GenericTreeModel):
     def removed(self, group, record):
         if (group is self.group
                 and (record.group is self.group
-                    or record.group.child_name == self.children_field)):
+                     or record.group.child_name == self.children_field)):
             path = record.get_index_path(self.group)
             self.row_deleted(path)
 
@@ -268,7 +268,6 @@ class AdaptModelGroup(GenericTreeModel):
 class TreeXMLViewParser(XMLViewParser):
 
     WIDGETS = {
-        'biginteger': Int,
         'binary': Binary,
         'boolean': Boolean,
         'callto': URL,
@@ -293,7 +292,7 @@ class TreeXMLViewParser(XMLViewParser):
         'time': Time,
         'timedelta': TimeDelta,
         'url': URL,
-        }
+    }
 
     def _parse_tree(self, node, attributes):
         for child in node.childNodes:
@@ -318,6 +317,7 @@ class TreeXMLViewParser(XMLViewParser):
 
         for affix in node.childNodes:
             affix_attrs = node_attributes(affix)
+            affix_attrs['visual'] = attributes.get('visual')
             if 'name' not in affix_attrs:
                 affix_attrs['name'] = attributes['name']
             if affix.tagName == 'prefix':
@@ -346,18 +346,8 @@ class TreeXMLViewParser(XMLViewParser):
 
         self.view.treeview.append_column(column)
 
-        if 'sum' in attributes:
-            text = attributes['sum'] + _(':')
-            label, sum_ = Gtk.Label(label=text), Gtk.Label()
-
-            hbox = Gtk.HBox()
-            hbox.pack_start(label, expand=True, fill=False, padding=2)
-            hbox.pack_start(sum_, expand=True, fill=False, padding=2)
-            hbox.show_all()
-            self.view.sum_box.pack_start(
-                hbox, expand=False, fill=False, padding=0)
-
-            self.view.sum_widgets.append((attributes['name'], sum_))
+        if 'optional' in attributes:
+            self.view.optionals.append(column)
 
     def _parse_button(self, node, attributes):
         button = Button(self.view, attributes)
@@ -401,26 +391,31 @@ class TreeXMLViewParser(XMLViewParser):
             hbox.pack_start(arrow_widget, expand=False, fill=False, padding=0)
             column.set_clickable(True)
         hbox.show()
-        column.set_widget(hbox)
+        if bool(int(attributes.get('sum', 0))):
+            vbox = Gtk.VBox(homogeneous=False, spacing=0)
+            vbox.pack_start(hbox, expand=False, fill=True, padding=0)
+            sum_ = Gtk.Label()
+            sum_.set_justify(Gtk.Justification.RIGHT)
+            sum_.show()
+            vbox.pack_start(sum_, expand=False, fill=True, padding=0)
+            self.view.sum_widgets.append((attributes['name'], sum_))
+            vbox.show()
+            widget = vbox
+        else:
+            widget = hbox
+        column.set_widget(widget)
         column.set_alignment(align)
 
     def _set_column_width(self, column, attributes):
         default_width = {
-            'integer': 60,
-            'biginteger': 60,
-            'float': 80,
-            'numeric': 80,
-            'timedelta': 100,
-            'date': 100,
-            'datetime': 100,
-            'time': 100,
+            'integer': 80,
             'selection': 90,
-            'char': 100,
+            'reference': 200,
             'one2many': 50,
             'many2many': 50,
-            'boolean': 20,
+            'boolean': 30,
             'binary': 200,
-            }
+        }
 
         screen = self.view.screen
         width = screen.tree_column_width[screen.model_name].get(column.name)
@@ -453,10 +448,12 @@ class ViewTree(View):
 
     def __init__(self, view_id, screen, xml, children_field):
         self.children_field = children_field
+        self.optionals = []
         self.sum_widgets = []
         self.sum_box = Gtk.HBox()
         self.treeview = None
         self._editable = bool(int(xml.getAttribute('editable') or 0))
+        self._creatable = bool(int(xml.getAttribute('creatable') or 1))
         if self._editable:
             self.treeview = EditableTreeView(self)
             grid_lines = Gtk.TreeViewGridLines.BOTH
@@ -469,17 +466,6 @@ class ViewTree(View):
 
         self.mnemonic_widget = self.treeview
 
-        # Add last column if necessary
-        for column in self.treeview.get_columns():
-            if column.get_expand():
-                break
-        else:
-            column = Gtk.TreeViewColumn()
-            column._type = 'fill'
-            column.name = None
-            column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
-            self.treeview.append_column(column)
-
         self.treeview.set_property('enable-grid-lines', grid_lines)
         self.treeview.set_fixed_height_mode(
             all(c.get_sizing() == Gtk.TreeViewColumnSizing.FIXED
@@ -489,6 +475,8 @@ class ViewTree(View):
         self.treeview.connect_after('row-activated', self.__sig_switch)
         if self.children_field:
             self.treeview.connect('test-expand-row', self.test_expand_row)
+            self.treeview.connect('row-expanded', self._row_expanded)
+            self.treeview.connect('row-collapsed', self._row_collapsed)
         self.treeview.set_rubber_banding(True)
 
         selection = self.treeview.get_selection()
@@ -513,10 +501,63 @@ class ViewTree(View):
 
         self.display()
 
+        if self.optionals:
+            if self.draggable:
+                column = self.treeview.get_columns()[0]
+            else:
+                column = Gtk.TreeViewColumn()
+                column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
+                column.name = None
+                column._type = 'optional'
+                self.treeview.insert_column(column, 0)
+            image = Gtk.Image()
+            image.set_from_pixbuf(
+                common.IconFactory.get_pixbuf('gnuhealth-menu'))
+            image.show()
+            column.set_widget(image)
+            column.set_fixed_width(25)
+            column.set_clickable(True)
+            column.connect('clicked', self.optional_menu)
+
+        # Add last column if necessary after display for updated visible
+        for column in self.treeview.get_columns():
+            if column.get_expand() and column.get_visible():
+                break
+        else:
+            column = Gtk.TreeViewColumn()
+            column._type = 'fill'
+            column.name = None
+            column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
+            self.treeview.append_column(column)
+
+    def optional_menu(self, column):
+        def toggle(menuitem, column):
+            column.set_visible(menuitem.get_active())
+            self.save_optional()
+
+        widget = column.get_widget()
+        menu = Gtk.Menu()
+        for optional in self.optionals:
+            menuitem = Gtk.CheckMenuItem(label=optional.get_title())
+            menuitem.set_active(optional.get_visible())
+            menuitem.connect('toggled', toggle, optional)
+            menu.add(menuitem)
+        popup(menu, widget)
+
+    def save_optional(self):
+        fields = {c.name: not c.get_visible() for c in self.optionals}
+        try:
+            RPCExecute(
+                'model', 'ir.ui.view_tree_optional', 'set_optional',
+                self.view_id, fields)
+        except RPCException:
+            pass
+        self.screen.tree_column_optional[self.view_id] = fields
+
     def get_column_widget(self, column):
         'Return the widget of the column'
         idx = [c for c in self.treeview.get_columns()
-            if c.name == column.name].index(column)
+               if c.name == column.name].index(column)
         return self.widgets[column.name][idx]
 
     def search_equal_func(self, model, column, key, iter_):
@@ -570,9 +611,11 @@ class ViewTree(View):
             if direction:
                 direction = direction.split(None, 1)[0]
                 direction = {
-                    'ASC': common.IconFactory.get_pixbuf('gnuhealth-arrow-down'),
-                    'DESC': common.IconFactory.get_pixbuf('gnuhealth-arrow-up'),
-                    }[direction]
+                    'ASC': common.IconFactory.get_pixbuf(
+                        'gnuhealth-arrow-down'),
+                    'DESC': common.IconFactory.get_pixbuf(
+                        'gnuhealth-arrow-up'),
+                }[direction]
         else:
             name, direction = None, None
 
@@ -619,12 +662,12 @@ class ViewTree(View):
         self.treeview.drag_source_set(
             Gdk.ModifierType.BUTTON1_MASK | Gdk.ModifierType.BUTTON3_MASK,
             [Gtk.TargetEntry.new(
-                    'MY_TREE_MODEL_ROW', Gtk.TargetFlags.SAME_WIDGET, 0)],
+                'MY_TREE_MODEL_ROW', Gtk.TargetFlags.SAME_WIDGET, 0)],
             Gdk.DragAction.MOVE)
 
         self.treeview.connect("drag-data-get", self.drag_data_get)
         self.treeview.connect('drag-data-received',
-            self.drag_data_received)
+                              self.drag_data_received)
         self.treeview.connect('drag-drop', self.drag_drop)
         self.treeview.connect('drag-data-delete', self.drag_data_delete)
 
@@ -633,7 +676,8 @@ class ViewTree(View):
         drag_column._type = 'drag'
         drag_column.name = None
         cell_pixbuf = Gtk.CellRendererPixbuf()
-        cell_pixbuf.props.pixbuf = common.IconFactory.get_pixbuf('gnuhealth-drag')
+        cell_pixbuf.props.pixbuf = common.IconFactory.get_pixbuf(
+            'gnuhealth-drag')
         drag_column.pack_start(cell_pixbuf, expand=False)
         self.treeview.insert_column(drag_column, 0)
 
@@ -645,13 +689,17 @@ class ViewTree(View):
     def editable(self):
         return self._editable and not self.screen.readonly
 
+    @property
+    def creatable(self):
+        return self.editable and self._creatable
+
     def get_fields(self, visible_only=False):
         return [col.name for col in self.treeview.get_columns()
-            if col.name and (not visible_only or col.get_visible())]
+                if col.name and (not visible_only or col.get_visible())]
 
     def get_buttons(self):
         return [b for b in self.state_widgets
-            if isinstance(b.renderer, CellRendererButton)]
+                if isinstance(b.renderer, CellRendererButton)]
 
     def on_keypress(self, widget, event):
         control_mask = Gdk.ModifierType.CONTROL_MASK
@@ -672,20 +720,22 @@ class ViewTree(View):
             self.record = model.get_value(iter_, 0)
             self.screen.switch_view('form')
             return True
+        return False
+
+    def _row_expanded(self, treeview, iter_, path):
+        # Force record_message
+        self.screen.current_record = self.screen.current_record
+
+    def _row_collapsed(self, treeview, iter_, path):
+        # Force record_message
+        self.screen.current_record = self.screen.current_record
+        model = treeview.get_model()
         iter_ = model.iter_children(iter_)
-        if not iter_:
-            return False
-        fields = [col.name for col in self.treeview.get_columns()
-                if col.name]
         while iter_:
             record = model.get_value(iter_, 0)
-            if not record.get_loaded(fields):
-                for field in fields:
-                    record[field]
-                    if record.exception:
-                        return True
+            if record.exception:
+                record.cancel()
             iter_ = model.iter_next(iter_)
-        return False
 
     def on_copy(self):
         for clipboard_type in [
@@ -704,8 +754,9 @@ class ViewTree(View):
                 continue
             widget = self.get_column_widget(col)
             values.append('"'
-                + str(widget.get_textual_value(record)).replace('"', '""')
-                + '"')
+                          + str(widget.get_textual_value(record)
+                                ).replace('"', '""')
+                          + '"')
         data.append('\t'.join(values))
         return
 
@@ -724,12 +775,12 @@ class ViewTree(View):
             text = clipboard.wait_for_text()
             if not text:
                 continue
-            data = [[unquote(v) for v in l.split('\t')]
-                for l in text.splitlines()]
+            data = [[unquote(v) for v in line.split('\t')]
+                    for line in text.splitlines()]
             break
         col = self.treeview.get_cursor()[1]
         columns = [c for c in self.treeview.get_columns()
-            if c.get_visible() and c.name]
+                   if c.get_visible() and c.name]
         if col in columns:
             idx = columns.index(col)
             columns = columns[idx:]
@@ -764,13 +815,13 @@ class ViewTree(View):
         self.screen.display(set_cursor=True)
 
     def drag_data_get(self, treeview, context, selection, target_id,
-            etime):
+                      etime):
         treeview.stop_emission_by_name('drag-data-get')
 
         def _func_sel_get(model, path, iter_, data):
             value = model.get_value(iter_, 0)
             data.append(json.dumps(
-                    value.get_path(model.group), separators=(',', ':')))
+                value.get_path(model.group), separators=(',', ':')))
         data = []
         treeselection = treeview.get_selection()
         treeselection.selected_foreach(_func_sel_get, data)
@@ -780,7 +831,7 @@ class ViewTree(View):
         return True
 
     def drag_data_received(self, treeview, context, x, y, selection,
-            info, etime):
+                           info, etime):
         treeview.stop_emission_by_name('drag-data-received')
         if self.attributes.get('sequence'):
             field = self.group.fields[self.attributes['sequence']]
@@ -927,14 +978,16 @@ class ViewTree(View):
             return True
         return False
 
-    def group_list_changed(self, group, signal):
+    def group_list_changed(self, group, action, *args):
         model = self.treeview.get_model()
         if model is not None:
-            if signal[0] == 'record-added':
-                model.added(group, signal[1])
-            elif signal[0] == 'record-removed':
-                model.removed(group, signal[1])
-        self.display()
+            if action == 'record-added':
+                record, pos = args
+                model.added(group, record)
+            elif action == 'record-removed':
+                record, pos = args
+                model.removed(group, record)
+        self.display(force=action == 'group-cleared')
 
     def __str__(self):
         return 'ViewList (%d)' % id(self)
@@ -964,7 +1017,7 @@ class ViewTree(View):
             model_name = self.screen.model_name
             try:
                 RPCExecute('model', 'ir.ui.view_tree_width', 'set_width',
-                    model_name, fields)
+                           model_name, fields)
             except RPCException:
                 pass
             self.screen.tree_column_width[model_name].update(fields)
@@ -992,7 +1045,7 @@ class ViewTree(View):
         previous_record = self.record
         if (previous_record
                 and (previous_record not in previous_record.group
-                    or previous_record.destroyed)):
+                     or previous_record.destroyed)):
             previous_record = None
 
         if tree_sel.get_mode() == Gtk.SelectionMode.SINGLE:
@@ -1079,25 +1132,34 @@ class ViewTree(View):
             domain.append(tab_domain)
         domain = simplify(domain)
         decoder = PYSONDecoder(self.screen.context)
+        tree_column_optional = self.screen.tree_column_optional.get(
+            self.view_id, {})
         for column in self.treeview.get_columns():
             name = column.name
             if not name:
                 continue
             widget = self.get_column_widget(column)
             widget.set_editable()
-            if decoder.decode(widget.attrs.get('tree_invisible', '0')):
+            if column.name in tree_column_optional:
+                optional = tree_column_optional[column.name]
+            else:
+                optional = bool(int(widget.attrs.get('optional', '0')))
+            invisible = decoder.decode(widget.attrs.get('tree_invisible', '0'))
+            if invisible or optional:
                 column.set_visible(False)
             elif name == self.screen.exclude_field:
                 column.set_visible(False)
-            else:
+            elif name in self.group.fields:
+                field = self.group.fields[name]
                 inv_domain = domain_inversion(domain, name)
                 if not isinstance(inv_domain, bool):
                     inv_domain = simplify(inv_domain)
-                unique, _, _ = unique_value(inv_domain)
+                unique, _, _ = unique_value(
+                    inv_domain, single_value=field._single_value)
                 column.set_visible(not unique or bool(self.children_field))
         if self.children_field:
             for i, column in enumerate(self.treeview.get_columns()):
-                if self.draggable and not i:
+                if (self.draggable or self.optionals) and not i:
                     continue
                 if column.get_visible():
                     self.treeview.set_expander_column(column)
@@ -1158,7 +1220,7 @@ class ViewTree(View):
                         '{}'.format(selected_sum or 0), True)
                     sum_ = locale.localize('{}'.format(sum_ or 0), True)
 
-                text = '%s / %s' % (selected_sum, sum_)
+                text = '%s\n%s' % (selected_sum, sum_)
             else:
                 text = '-'
             label.set_text(text)
@@ -1175,7 +1237,7 @@ class ViewTree(View):
             if path[:-1]:
                 self.treeview.expand_to_path(Gtk.TreePath(path[:-1]))
             self.treeview.scroll_to_cell(path, focus_column,
-                use_align=False)
+                                         use_align=False)
             current_path = self.treeview.get_cursor()[0]
             selected_path = \
                 self.treeview.get_selection().get_selected_rows()[1]
@@ -1189,8 +1251,7 @@ class ViewTree(View):
             records.append(model.get_value(iter_, 0))
         records = []
         sel = self.treeview.get_selection()
-        if sel is not None:
-            sel.selected_foreach(_func_sel_get, records)
+        sel.selected_foreach(_func_sel_get, records)
         return records
 
     def get_selected_paths(self):
