@@ -5,37 +5,37 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2022 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from itertools import chain
 import gettext
+from itertools import chain
 
 from gi.repository import Gdk, Gtk, Pango
 
-from .infobar import InfoBar
 import gnuhealth.common as common
-from gnuhealth.common import GNUHEALTH_ICON, MODELNAME
+from gnuhealth.common import MODELNAME, GNUHEALTH_ICON
 from gnuhealth.common.domain_parser import quote
 from gnuhealth.common.underline import set_underline
 from gnuhealth.common.widget_style import widget_class
 from gnuhealth.gui import Main
 from gnuhealth.gui.window.nomodal import NoModal
 
+from .infobar import InfoBar
+
 _ = gettext.gettext
 
 
 class WinForm(NoModal, InfoBar):
     "Form window"
 
-    def __init__(self, screen, callback, view_type='form',
-            new=False, many=0, domain=None, context=None,
-            save_current=False, title='', rec_name=None):
+    def __init__(self, screen, callback=None, view_type='form',
+                 new=False, many=0, domain=None, context=None,
+                 save_current=False, title='', defaults=None):
         tooltips = common.Tooltips()
         NoModal.__init__(self)
         self.screen = screen
@@ -49,8 +49,8 @@ class WinForm(NoModal, InfoBar):
             if title:
                 breadcrumb.append(title)
             self.title = ' › '.join(chain(
-                    (common.ellipsize(x, 30) for x in breadcrumb[-3:-1]),
-                    breadcrumb[-1:]))
+                (common.ellipsize(x, 30) for x in breadcrumb[-3:-1]),
+                breadcrumb[-1:]))
             if len(breadcrumb) > 3:
                 self.title = '... › ' + self.title
         else:
@@ -64,7 +64,7 @@ class WinForm(NoModal, InfoBar):
             self.destroy()
             return
         if new:
-            self.screen.new(rec_name=rec_name)
+            self.screen.new(defaults=defaults)
         self.win = Gtk.Dialog(
             title=_('Link'), transient_for=self.parent,
             destroy_with_parent=True)
@@ -81,17 +81,18 @@ class WinForm(NoModal, InfoBar):
         self.accel_group = Gtk.AccelGroup()
         self.win.add_accel_group(self.accel_group)
 
-        readonly = self.screen.readonly or self.screen.group.readonly
+        readonly = self.screen.group.readonly
 
         self.but_ok = None
         self.but_new = None
 
         self._initial_value = None
-        if view_type == 'form':
+        self.view_type = view_type
+        if view_type == 'form' and not readonly:
             if new:
                 label, icon = _("Delete"), 'gnuhealth-delete'
             else:
-                label, icon = _("Cancel"), 'gnuhealth-cancel'
+                label, icon = _("Discard changes"), 'gnuhealth-cancel'
                 record = self.screen.current_record
                 self._initial_value = record.get_on_change_value()
                 if record.parent and record.parent_name in record.group.fields:
@@ -101,35 +102,47 @@ class WinForm(NoModal, InfoBar):
             self.but_cancel = self.win.add_button(
                 set_underline(label), Gtk.ResponseType.CANCEL)
             self.but_cancel.set_image(common.IconFactory.get_image(
-                    icon, Gtk.IconSize.BUTTON))
+                icon, Gtk.IconSize.BUTTON))
             self.but_cancel.set_always_show_image(True)
 
         if new and self.many:
+            if self.save_current and not readonly:
+                label = _("Save and New")
+            else:
+                label = _("Add and New")
             self.but_new = self.win.add_button(
-                set_underline(_("New")), Gtk.ResponseType.ACCEPT)
+                set_underline(label), Gtk.ResponseType.ACCEPT)
             self.but_new.set_image(common.IconFactory.get_image(
-                    'gnuhealth-create', Gtk.IconSize.BUTTON))
+                'gnuhealth-create', Gtk.IconSize.BUTTON))
             self.but_new.set_always_show_image(True)
-            self.but_new.set_accel_path('<gnuhealth>/Form/New', self.accel_group)
+            self.but_new.set_accel_path(
+                '<from gnuhealth>/Form/New', self.accel_group)
 
-        if self.save_current:
-            self.but_ok = Gtk.Button(label=_('_Save'), use_underline=True)
+        if self.save_current and not readonly:
+            self.but_ok = Gtk.Button(
+                label=set_underline(_("Save")), use_underline=True)
             self.but_ok.set_image(common.IconFactory.get_image(
-                    'gnuhealth-save', Gtk.IconSize.BUTTON))
+                'gnuhealth-save', Gtk.IconSize.BUTTON))
             self.but_ok.set_always_show_image(True)
-            self.but_ok.set_accel_path('<gnuhealth>/Form/Save', self.accel_group)
+            self.but_ok.set_accel_path(
+                '<from gnuhealth>/Form/Save', self.accel_group)
             self.but_ok.set_can_default(True)
             self.but_ok.show()
             self.win.add_action_widget(self.but_ok, Gtk.ResponseType.OK)
-            if not new:
-                self.but_ok.props.sensitive = False
         else:
+            if readonly or view_type == 'tree':
+                label = _("Close")
+            elif new:
+                label = _("Add")
+            else:
+                label = _("Apply changes")
             self.but_ok = self.win.add_button(
-                set_underline(_("OK")), Gtk.ResponseType.OK)
+                set_underline(label), Gtk.ResponseType.OK)
             self.but_ok.set_image(common.IconFactory.get_image(
-                    'gnuhealth-ok', Gtk.IconSize.BUTTON))
+                'gnuhealth-ok', Gtk.IconSize.BUTTON))
             self.but_ok.set_always_show_image(True)
-        self.but_ok.add_accelerator('clicked', self.accel_group,
+        self.but_ok.add_accelerator(
+            'clicked', self.accel_group,
             Gdk.KEY_Return, Gdk.ModifierType.CONTROL_MASK,
             Gtk.AccelFlags.VISIBLE)
         self.win.set_default_response(Gtk.ResponseType.OK)
@@ -177,7 +190,7 @@ class WinForm(NoModal, InfoBar):
             tooltips.set_tip(but_switch, _('Switch'))
             but_switch.connect('clicked', self.switch_view)
             but_switch.add(common.IconFactory.get_image(
-                    'gnuhealth-switch', Gtk.IconSize.SMALL_TOOLBAR))
+                'gnuhealth-switch', Gtk.IconSize.SMALL_TOOLBAR))
             but_switch.set_relief(Gtk.ReliefStyle.NONE)
             hbox.pack_start(but_switch, expand=False, fill=False, padding=0)
 
@@ -185,7 +198,7 @@ class WinForm(NoModal, InfoBar):
             tooltips.set_tip(self.but_pre, _('Previous'))
             self.but_pre.connect('clicked', self._sig_previous)
             self.but_pre.add(common.IconFactory.get_image(
-                    'gnuhealth-back', Gtk.IconSize.SMALL_TOOLBAR))
+                'gnuhealth-back', Gtk.IconSize.SMALL_TOOLBAR))
             self.but_pre.set_relief(Gtk.ReliefStyle.NONE)
             hbox.pack_start(self.but_pre, expand=False, fill=False, padding=0)
 
@@ -196,7 +209,7 @@ class WinForm(NoModal, InfoBar):
             tooltips.set_tip(self.but_next, _('Next'))
             self.but_next.connect('clicked', self._sig_next)
             self.but_next.add(common.IconFactory.get_image(
-                    'gnuhealth-forward', Gtk.IconSize.SMALL_TOOLBAR))
+                'gnuhealth-forward', Gtk.IconSize.SMALL_TOOLBAR))
             self.but_next.set_relief(Gtk.ReliefStyle.NONE)
             hbox.pack_start(self.but_next, expand=False, fill=False, padding=0)
 
@@ -215,7 +228,7 @@ class WinForm(NoModal, InfoBar):
                 tooltips.set_tip(self.but_add, _('Add'))
                 self.but_add.connect('clicked', self._sig_add)
                 self.but_add.add(common.IconFactory.get_image(
-                        'gnuhealth-add', Gtk.IconSize.SMALL_TOOLBAR))
+                    'gnuhealth-add', Gtk.IconSize.SMALL_TOOLBAR))
                 self.but_add.set_relief(Gtk.ReliefStyle.NONE)
                 hbox.pack_start(
                     self.but_add, expand=False, fill=False, padding=0)
@@ -226,7 +239,7 @@ class WinForm(NoModal, InfoBar):
                 tooltips.set_tip(self.but_remove, _('Remove <Del>'))
                 self.but_remove.connect('clicked', self._sig_remove, True)
                 self.but_remove.add(common.IconFactory.get_image(
-                        'gnuhealth-remove', Gtk.IconSize.SMALL_TOOLBAR))
+                    'gnuhealth-remove', Gtk.IconSize.SMALL_TOOLBAR))
                 self.but_remove.set_relief(Gtk.ReliefStyle.NONE)
                 hbox.pack_start(
                     self.but_remove, expand=False, fill=False, padding=0)
@@ -240,7 +253,7 @@ class WinForm(NoModal, InfoBar):
             tooltips.set_tip(self.but_new, _('Create a new record <F3>'))
             self.but_new.connect('clicked', self._sig_new)
             self.but_new.add(common.IconFactory.get_image(
-                    'gnuhealth-create', Gtk.IconSize.SMALL_TOOLBAR))
+                'gnuhealth-create', Gtk.IconSize.SMALL_TOOLBAR))
             self.but_new.set_relief(Gtk.ReliefStyle.NONE)
             hbox.pack_start(self.but_new, expand=False, fill=False, padding=0)
             if not access['create'] or readonly:
@@ -250,7 +263,7 @@ class WinForm(NoModal, InfoBar):
             tooltips.set_tip(self.but_del, _('Delete selected record <Del>'))
             self.but_del.connect('clicked', self._sig_remove, False)
             self.but_del.add(common.IconFactory.get_image(
-                    'gnuhealth-delete', Gtk.IconSize.SMALL_TOOLBAR))
+                'gnuhealth-delete', Gtk.IconSize.SMALL_TOOLBAR))
             self.but_del.set_relief(Gtk.ReliefStyle.NONE)
             hbox.pack_start(self.but_del, expand=False, fill=False, padding=0)
             if not access['delete'] or readonly:
@@ -258,10 +271,10 @@ class WinForm(NoModal, InfoBar):
 
             self.but_undel = Gtk.Button()
             tooltips.set_tip(self.but_undel,
-                _('Undelete selected record <Ins>'))
+                             _('Undelete selected record <Ins>'))
             self.but_undel.connect('clicked', self._sig_undelete)
             self.but_undel.add(common.IconFactory.get_image(
-                    'gnuhealth-undo', Gtk.IconSize.SMALL_TOOLBAR))
+                'gnuhealth-undo', Gtk.IconSize.SMALL_TOOLBAR))
             self.but_undel.set_relief(Gtk.ReliefStyle.NONE)
             hbox.pack_start(
                 self.but_undel, expand=False, fill=False, padding=0)
@@ -285,20 +298,13 @@ class WinForm(NoModal, InfoBar):
 
         scroll.add(self.screen.screen_container.alternate_viewport)
 
-        self.create_info_bar()
         self.win.vbox.pack_start(
-            self.info_bar, expand=False, fill=True, padding=0)
+            self.create_info_bar(), expand=False, fill=True, padding=0)
 
+        self.screen.windows.append(self)
         if view_type == 'tree':
-            self.screen.signal_connect(self, 'record-message', self._sig_label)
             self.screen.screen_container.alternate_viewport.connect(
-                    'key-press-event', self.on_keypress)
-
-        if self.save_current and not new:
-            self.screen.signal_connect(self, 'record-message',
-                self.activate_save)
-            self.screen.signal_connect(self, 'record-modified',
-                self.activate_save)
+                'key-press-event', self.on_keypress)
 
         self.register()
         self.show()
@@ -353,37 +359,40 @@ class WinForm(NoModal, InfoBar):
             if result:
                 ids = [x[0] for x in result]
                 self.screen.load(ids, modified=True)
-                self.screen.display(res_id=ids[0])
             self.screen.set_cursor()
             self.wid_text.set_text('')
 
         win = WinSearch(model_name, callback, sel_multi=True,
-            context=self.context, domain=domain)
+                        context=self.context, domain=domain)
         win.screen.search_filter(quote(value))
         win.show()
 
-    def _sig_label(self, screen, signal_data):
+    def record_message(self, position, size, *args):
+        if self.view_type != 'tree':
+            return
         name = '_'
-        access = common.MODELACCESS[screen.model_name]
+        access = common.MODELACCESS[self.screen.model_name]
         deletable = True
-        if screen.current_record:
-            deletable = screen.current_record.deletable
-        readonly = screen.group.readonly
-        if signal_data[0] >= 1:
-            name = str(signal_data[0])
+        if self.screen.current_record:
+            deletable = self.screen.current_record.deletable
+        readonly = self.screen.group.readonly
+        if position >= 1:
+            name = str(position)
             if self.domain is not None:
                 self.but_remove.set_sensitive(True)
-            if signal_data[0] < signal_data[1]:
+            if position < size:
                 self.but_next.set_sensitive(True)
             else:
                 self.but_next.set_sensitive(False)
-            if signal_data[0] > 1:
+            if position > 1:
                 self.but_pre.set_sensitive(True)
             else:
                 self.but_pre.set_sensitive(False)
-            if access['delete'] and not readonly and deletable:
-                self.but_del.set_sensitive(True)
-                self.but_undel.set_sensitive(True)
+            self.but_del.set_sensitive(bool(
+                not readonly
+                and access['delete']
+                and deletable))
+            self.but_undel.set_sensitive(bool(not readonly))
         else:
             self.but_del.set_sensitive(False)
             self.but_undel.set_sensitive(False)
@@ -391,16 +400,11 @@ class WinForm(NoModal, InfoBar):
             self.but_pre.set_sensitive(False)
             if self.domain is not None:
                 self.but_remove.set_sensitive(False)
-        line = '(%s/%s)' % (name, signal_data[1])
+        line = '(%s/%s)' % (name, position)
         self.label.set_text(line)
 
-    def activate_save(self, *args):
-        modified = self.screen.modified()
-        # Keep sensible as change could have been trigger by a Many2One edition
-        sensitive = modified or self.but_ok.props.sensitive
-        self.but_ok.props.sensitive = sensitive
-        self.win.set_default_response(
-            Gtk.ResponseType.OK if sensitive else Gtk.ResponseType.CANCEL)
+    def record_modified(self, *args):
+        self.info_bar_refresh()
 
     def close(self, widget):
         widget.stop_emission_by_name('close')
@@ -433,12 +437,12 @@ class WinForm(NoModal, InfoBar):
                             if record:
                                 validate = record.pre_validate()
             if not validate:
-                self.message_info(self.screen.invalid_message(),
-                    Gtk.MessageType.ERROR)
+                self.info_bar_add(
+                    self.screen.invalid_message(), Gtk.MessageType.ERROR)
                 self.screen.set_cursor()
                 self.screen.display()
                 return
-            self.message_info()
+            self.info_bar_clear()
             if response_id == Gtk.ResponseType.ACCEPT:
                 self.new()
                 return
@@ -453,13 +457,14 @@ class WinForm(NoModal, InfoBar):
             elif record.modified:
                 record.cancel()
                 record.reload()
-                record.signal('record-changed')
+                record.set_modified()
             if added:
                 record.modified_fields.setdefault('id')
             result = False
         else:
-            result = response_id not in cancel_responses
-        self.callback(result)
+            result = (response_id not in cancel_responses) and not readonly
+        if self.callback:
+            self.callback(result)
         self.destroy()
 
     def new(self):
@@ -473,12 +478,15 @@ class WinForm(NoModal, InfoBar):
             self.win.set_default_response(Gtk.ResponseType.OK)
 
     def destroy(self):
+        try:
+            self.screen.windows.remove(self)
+        except ValueError:
+            pass
         self.screen.screen_container.alternate_view = False
         viewport = self.screen.screen_container.alternate_viewport
         if viewport and viewport.get_parent():
             viewport.get_parent().remove(viewport)
         self.screen.switch_view(view_type=self.prev_view.view_type)
-        self.screen.signal_unconnect(self)
         if getattr(self, 'win', None):
             self.win.destroy()
         NoModal.destroy(self)
