@@ -5,29 +5,28 @@
 #       The GNUHealth HMIS client based on the Tryton GTK Client        #
 #########################################################################
 #
-# SPDX-FileCopyrightText: 2008-2021 The Tryton Community <info@tryton.org>
-# SPDX-FileCopyrightText: 2017-2023 GNU Health Community <info@gnuhealth.org>
+# SPDX-FileCopyrightText: 2008-2024 The Tryton Community <info@tryton.org>
+# SPDX-FileCopyrightText: 2017-2024 GNU Health Community <info@gnuhealth.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 
-
 # This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import logging
 import gettext
+import logging
 
 from gi.repository import Gdk, Gtk, Pango
 
-from gnuhealth.signal_event import SignalEvent
 import gnuhealth.common as common
-from gnuhealth.gui.window.view_form.screen import Screen
-from gnuhealth.gui import Main
-from gnuhealth.exceptions import GNUHealthServerError
-from gnuhealth.gui.window.nomodal import NoModal
+from gnuhealth.common import (
+    GNUHEALTH_ICON, RPCContextReload, RPCException, RPCExecute)
 from gnuhealth.common.button import Button
-from gnuhealth.common import RPCExecute, RPCException, RPCContextReload
-from gnuhealth.common import GNUHEALTH_ICON
 from gnuhealth.common.widget_style import widget_class
+from gnuhealth.exceptions import GNUHealthServerError
+from gnuhealth.gui import Main
+from gnuhealth.gui.window.nomodal import NoModal
+from gnuhealth.gui.window.view_form.screen import Screen
+
 from .infobar import InfoBar
 from .tabcontent import TabContent
 
@@ -98,7 +97,7 @@ class Wizard(InfoBar):
         if self.screen:
             data = {
                 self.screen_state: self.screen.get_on_change_value(),
-                }
+            }
         else:
             data = {}
 
@@ -107,7 +106,7 @@ class Wizard(InfoBar):
                 result = result()
             except RPCException as rpc_exception:
                 if (not isinstance(rpc_exception.exception,
-                        GNUHealthServerError)
+                                   GNUHealthServerError)
                         or not self.screen):
                     self.state = self.end_state
                     self.end()
@@ -120,8 +119,11 @@ class Wizard(InfoBar):
                 self.update(view['fields_view'], view['buttons'])
 
                 self.screen.new(default=False)
-                self.screen.current_record.set_default(view['defaults'])
-                self.update_buttons(self.screen.current_record)
+                if 'defaults' in view:
+                    self.screen.current_record.set_default(view['defaults'])
+                if 'values' in view:
+                    self.screen.current_record.set(view['values'])
+                self.update_buttons()
                 self.screen.set_cursor()
 
                 self.screen_state = view['state']
@@ -135,7 +137,7 @@ class Wizard(InfoBar):
                             ('direct_print', self.direct_print),
                             ('email_print', self.email_print),
                             ('email', self.email),
-                            ]:
+                    ]:
                         action[0].setdefault(k, v)
                     context = self.context.copy()
                     # Remove wizard keys added by run
@@ -152,7 +154,7 @@ class Wizard(InfoBar):
             self.__processing = False
 
         RPCExecute('wizard', self.action, 'execute', self.session_id, data,
-            self.state, context=ctx, callback=callback)
+                   self.state, context=ctx, callback=callback)
 
     def destroy(self, action=None):
         if self.screen:
@@ -165,11 +167,11 @@ class Wizard(InfoBar):
                 callback()
         try:
             RPCExecute('wizard', self.action, 'delete', self.session_id,
-                process_exception=False, callback=end_callback)
+                       process_exception=False, callback=end_callback)
         except Exception:
             logger.warn(
-                _("Unable to delete wizard %s") % self.session_id,
-                exc_info=True)
+                "Unable to delete session %s of wizard %s",
+                self.session_id, self.action, exc_info=True)
 
     def clean(self):
         for widget in self.widget.get_children():
@@ -184,10 +186,10 @@ class Wizard(InfoBar):
         if (button_attrs.get('validate', True)
                 and not self.screen.current_record.validate()):
             self.screen.display(set_cursor=True)
-            self.message_info(
+            self.info_bar_add(
                 self.screen.invalid_message(), Gtk.MessageType.ERROR)
             return
-        self.message_info()
+        self.info_bar_clear()
         self.state = state
         self.process()
 
@@ -199,10 +201,12 @@ class Wizard(InfoBar):
         button.show()
         return button
 
-    def _record_changed(self, screen, record):
-        self.update_buttons(record)
+    def record_modified(self):
+        self.update_buttons()
+        self.info_bar_refresh()
 
-    def update_buttons(self, record):
+    def update_buttons(self):
+        record = self.screen.current_record
         for button in self.states.values():
             button.state_set(record)
 
@@ -211,12 +215,14 @@ class Wizard(InfoBar):
         for button in buttons:
             self._get_button(button)
 
+        if self.screen:
+            self.screen.windows.remove(self)
+
         self.screen = Screen(view['model'], mode=[], context=self.context)
         self.screen.add_view(view)
         self.screen.switch_view()
         self.screen.widget.show()
-        self.screen.signal_connect(self, 'group-changed',
-            self._record_changed)
+        self.screen.windows.append(self)
 
         title = Gtk.Label(
             label=common.ellipsize(self.name, 80),
@@ -252,12 +258,11 @@ class Wizard(InfoBar):
         self.widget.pack_start(
             self.scrolledwindow, expand=True, fill=True, padding=0)
 
-        self.create_info_bar()
         self.widget.pack_start(
-            self.info_bar, expand=False, fill=True, padding=0)
+            self.create_info_bar(), expand=False, fill=True, padding=0)
 
 
-class WizardForm(Wizard, TabContent, SignalEvent):
+class WizardForm(Wizard, TabContent):
     "Wizard"
 
     def __init__(self, name=''):
@@ -409,6 +414,8 @@ class WizardDialog(Wizard, NoModal):
         return True
 
     def show(self):
+        if not self.screen:
+            return
         view = self.screen.current_view
         if view.view_type == 'form':
             expand = False
@@ -425,11 +432,15 @@ class WizardDialog(Wizard, NoModal):
             width, height = self.default_size()
         else:
             width, height = -1, -1
-        self.dia.set_default_size(max(200, width), height)
-        width, height = self.dia.get_default_size()
-        if width > 0 and height > 0:
-            self.dia.resize(*self.dia.get_default_size())
+        self.dia.set_default_size(width, height)
+        # reshow with initial size
+        self.dia.hide()
+        self.dia.unrealize()
         self.dia.show()
+        width, height = self.dia.get_size()
+        screen = self.dia.get_screen()
+        self.dia.resize(
+            min(width, screen.width()), min(height, screen.height()))
 
     def hide(self):
         self.dia.hide()
